<?php

namespace ShopWP\API;

use ShopWP\Utils;

if (!defined('ABSPATH')) {
	exit;
}

class GraphQL {

    public $DB_Settings_Connection;
    public $plugin_settings;

    public function __construct($DB_Settings_Connection, $plugin_settings) {
        $this->DB_Settings_Connection   = $DB_Settings_Connection;
        $this->plugin_settings          = $plugin_settings;
    }

    public function graphql_storefront_api_endpoint($domain) {
        return 'https://' . $domain . '/api/' . SHOPWP_STOREFRONT_GRAPHQL_API_VERSION . '/graphql.json';
    }

    public function graphql_admin_api_endpoint($domain) {
        return 'https://' . $domain . '/admin/api/' . SHOPWP_ADMIN_GRAPHQL_API_VERSION . '/graphql.json';
    }

    public function graphql_storefront_api_headers() {

        return [
            'X-Shopify-Storefront-Access-Token' => $this->DB_Settings_Connection->get_storefront_access_token(),
            'Content-type' => 'application/json',
            'Accept' => "application/json"
        ];
    }

    public function graphql_admin_api_headers() {
        return [
            'X-Shopify-Access-Token' => $this->DB_Settings_Connection->get_admin_api_password(),
            'Content-type' => 'application/json',
        ];
    }

    public function post($endpoint, $options) {
        return \wp_remote_post($endpoint, $options);
    }

    public function graph_post_options($query, $type) {
        return [
            'headers' => $type === 'storefront' ? $this->graphql_storefront_api_headers() : $this->graphql_admin_api_headers(),
            'body' => json_encode($query),
            'timeout' => 60
        ];
    }

    public function graphql_api_request($query, $type = 'storefront', $access_keys = false) {

        $domain = $this->DB_Settings_Connection->get_domain();

        if (empty($domain)) {
            return Utils::wp_error(__('No Shopify connection found. Please try reconnecting your Shopify store.', 'shopwp'));
        }

        if ($type === 'storefront') {
            $endpoint = $this->graphql_storefront_api_endpoint($domain);

        } else if ($type === 'admin') {
            $endpoint = $this->graphql_admin_api_endpoint($domain);
        }
    
        $options = $this->graph_post_options($query, $type);

        return $this->return_response(
            $this->post(
                $endpoint, 
                $options
            ),
            $access_keys
        );
    }

    public function return_response($response, $access_keys = false) {

        $response_headers    = wp_remote_retrieve_headers($response);
        $response_code       = wp_remote_retrieve_response_code($response);
        $response_message    = wp_remote_retrieve_response_message($response);
        $body                = json_decode(wp_remote_retrieve_body($response));

        if (is_wp_error($response)) {
            return $response;
        }

        if ( 200 != $response_code && !empty( $response_message ) ) {
            return Utils::wp_error($response_message);

        } elseif ( 200 != $response_code ) {
            return Utils::wp_error(__('Unknown error occurred while calling Shopify', 'shopwp'));

        } elseif (property_exists($body, 'errors')) {
            return Utils::wp_error($body->errors[0]->message);

        } else {

            if ($access_keys) {

                if (empty($access_keys['mutation_key'])) {
                    return $body->data->{$access_keys['data_key']};
                }

                $raw_data = $body->data->{$access_keys['mutation_key']};

                if (!empty($raw_data->{$access_keys['user_errors_key']})) {
                    return Utils::wp_error($raw_data->{$access_keys['user_errors_key']}[0]->message);
                }

                return $raw_data->{$access_keys['data_key']};

            }

            return $body->data;

        }

    }

    public function normalize_buyer_identity($buyerIdentity, $params = [], $remove_others = false) {

        $default_language   = strtoupper($this->plugin_settings['general']['language_code']);
        $default_country    = strtoupper($this->plugin_settings['general']['country_code']);

        if (empty($buyerIdentity)) {
            
            $params['buyerIdentity'] = [
                'language'  => $default_language,
                'country'   => $default_country
            ];

            return $params;
        }

        if (empty($buyerIdentity['country'])) {
            $params['buyerIdentity']['country'] = $default_country;

        } else {
            $params['buyerIdentity']['country'] = strtoupper($buyerIdentity['country']);
        }

        if (empty($buyerIdentity['language'])) {
            $params['buyerIdentity']['language'] = $default_language;

        } else {
            $params['buyerIdentity']['language'] = strtoupper($buyerIdentity['language']);
        }

        if (empty($buyerIdentity['token'])) {
            $params['buyerIdentity']['customerAccessToken'] = '';

        } else {
            $params['buyerIdentity']['customerAccessToken'] = $buyerIdentity['token'];
        }

        if (empty($buyerIdentity['email'])) {
            $params['buyerIdentity']['email'] = '';

        } else {
            $params['buyerIdentity']['email'] = $buyerIdentity['email'];
        }

        if (empty($buyerIdentity['phone'])) {
            $params['buyerIdentity']['phone'] = '';

        } else {
            $params['buyerIdentity']['phone'] = $buyerIdentity['phone'];
        }


        if (empty($params['buyerIdentity']['customerAccessToken'])) {
            unset($params['buyerIdentity']['customerAccessToken']);
        }

        if (empty($params['buyerIdentity']['email'])) {
            unset($params['buyerIdentity']['email']);
        }

        if (empty($params['buyerIdentity']['phone'])) {
            unset($params['buyerIdentity']['phone']);
        }

        $params['buyerIdentity']['countryCode'] = $params['buyerIdentity']['country'];

        if ($remove_others) {
            unset($params['buyerIdentity']['country']);
            unset($params['buyerIdentity']['language']);
        }

        if (!empty($params['metafields'])) {
            
            if (!empty($params['metafields']['variants'])) {
                $metafields = json_encode($params['metafields']['variants']);

            } else {
                $metafields = json_encode($params['metafields']);
            }

            if (empty($metafields)) {
                return $params;
            }

            $metafields  = \str_replace('"namespace"', 'namespace', $metafields);
            $metafields  = \str_replace('"key"', 'key', $metafields);
            
            $metafields = 'metafields(identifiers: ' . $metafields . ' ) { 
                            type 
                            namespace 
                            key 
                            value
                            }';

            $params['metafields'] = $metafields;

        }

        if (isset($params['metafields']) && empty($params['metafields'])) {
            unset($params['metafields']);
        }

        return $params;

    }

}
