<?php

namespace ShopWP\API\Items;

use ShopWP\Options;
use ShopWP\Utils;

if (!defined('ABSPATH')) {
    exit();
}

class Cart extends \ShopWP\API
{

    public $plugin_settings;
    public $Storefront_Cart;
    public $DB_Settings_General;

    public function __construct($plugin_settings, $Storefront_Cart, $DB_Settings_General) {
        $this->plugin_settings      = $plugin_settings;
        $this->Storefront_Cart      = $Storefront_Cart;
        $this->DB_Settings_General  = $DB_Settings_General;
    }

    public function handle_apply_discount($request) {

        $cartId                     = $request->get_param('cartId');
        $existingDiscountCodes      = $request->get_param('existingDiscountCodes');
        $buyerIdentity              = $request->get_param('buyerIdentity');
        $metafields                 = $request->get_param('metafields');
        $discountToChange           = $request->get_param('discountToChange');
        $isRemoving                 = $request->get_param('isRemoving');
        $existingDiscountCodes      = array_values(array_filter(array_unique($existingDiscountCodes)));

        if ($isRemoving) {

            $left_overs = array_filter($existingDiscountCodes, function($discount) use($discountToChange) {
                return $discount['code'] !== $discountToChange;
            });

            $final_d = array_map(function($discount) {
                return $discount['code'];
            }, $left_overs);

        } else {

            $final_d = array_map(function($discount) {
                return $discount['code'];
            }, $existingDiscountCodes);

            $final_d[] = $discountToChange;

        }

        $final_d = array_values(array_filter(array_unique($final_d)));

        $cartData = [
            'cartId'            => !empty($cartId) ? $cartId : false,
            'discountCodes'     => empty($final_d) ? '' : $final_d,
            'metafields'        => !empty($metafields) ? $metafields : false
        ];

        $cartData = $this->Storefront_Cart->GraphQL->normalize_buyer_identity($buyerIdentity, $cartData);
        $response = $this->Storefront_Cart->api_apply_discount($cartData);

        if (is_wp_error($response)) {
            return wp_send_json_error($response);
        }

        $filtered           = array_filter($response->discountCodes);
        $applicable_codes   = $this->only_applicable_discounts($filtered);

        /*
        
        Only run these checks when adding a discount code
        
        */
        if (!$isRemoving) {

            // Is the added code found in list of applicable codes?
            $found_code = array_filter($applicable_codes, function($applicable_code) use($discountToChange) {
                return $applicable_code->code === $discountToChange;
            });

            // If the user entered an invalid code, or the list of applied codes is empty ...
            if (empty($applicable_codes) || empty($found_code)) {
                return wp_send_json_error(__('The discount code entered is either inactive or not valid for some products in your cart. Please try a different code.', 'shopwp'));
            }
        }

        return \wp_send_json_success($response);

    }

    public function only_applicable_discounts($codes) {

        if (empty($codes)) {
            return [];
        }

        return array_filter($codes, function($code) {
            return $code->applicable;
        });
    }

    public function find_lineitem_options_from_lineitem_id($lineitem_options, $lineitem_id) {

        if (empty($lineitem_options)) {
            return false;
        }

        return array_filter($lineitem_options, function($lineitem_option) use($lineitem_id) {
            return $lineitem_option['variantId'] === $lineitem_id;
        });

    }

    public function handle_update_cart_attributes($request) {

        $cartId             = $request->get_param('cartId');
        $attributes         = $request->get_param('attributes');
        $buyerIdentity      = $request->get_param('buyerIdentity');
        $metafields         = $request->get_param('metafields');

        if (empty($cartId)) {
            return wp_send_json_error(__('No cart id found. Please make sure you\'re passing in a valid cart id when updating cart attributes', 'shopwp'));
        }

        $cartData = [
            'cartId'        => $cartId,
            'attributes'    => !empty($attributes) ? $attributes : [],
            'metafields'    => !empty($metafields) ? $metafields : false
        ];

        $cartData = $this->Storefront_Cart->GraphQL->normalize_buyer_identity($buyerIdentity, $cartData);

        $response = $this->Storefront_Cart->api_update_cart_attributes($cartData);

        if (is_wp_error($response)) {
            return wp_send_json_error($response);
        }

        return \wp_send_json_success($response);
                
    }

    public function handle_update_buyer_identity($request) {
        
        $buyerIdentity      = $request->get_param('buyerIdentity');
        $cartId             = $request->get_param('cartId');
        $metafields         = $request->get_param('metafields');

        if (empty($cartId)) {
            return \wp_send_json_error(__('No cart id found. Please make sure you\'re passing in a valid cart id when updating buyer identity', 'shopwp'));
        }

        $cartData = [
            'cartId'        => $cartId,
            'metafields'    => !empty($metafields) ? $metafields : false
        ];

        $cartData = $this->Storefront_Cart->GraphQL->normalize_buyer_identity($buyerIdentity, $cartData);

        $language = $cartData['buyerIdentity']['language'];
        
        unset($cartData['buyerIdentity']['country']);
        unset($cartData['buyerIdentity']['language']);

        $response = $this->Storefront_Cart->api_update_buyer_identity($cartData, $language);

        if (is_wp_error($response)) {
            return \wp_send_json_error($response->get_error_message());
        }

        return \wp_send_json_success($response);
                
    }

    public function create_cart($cartData, $buyerIdentity) {
        $response = $this->Storefront_Cart->api_create_cart($cartData, $buyerIdentity['language'], $buyerIdentity['country']);

        if (empty($response)) {
            return Utils::wp_error(__('Tried creating cart, but did not receive a valid response.', 'shopwp'));
        }

        if (\is_wp_error($response)) {
            return $response;
        }

        return $response;
    }

    public function handle_create_cart($request) {

        $lines              = $request->get_param('lines');
        $note               = $request->get_param('note');
        $attributes         = $request->get_param('attributes');
        $discountCodes      = $request->get_param('discountCodes');
        $buyerIdentity      = $request->get_param('buyerIdentity');

        $lines              = empty($lines) ? [] : $this->sanitize_lineitems($lines);
        $note               = empty($note) ? '' : $note;
        $attributes         = empty($attributes) ? [] : $attributes;
        $discountCodes      = empty($discountCodes) ? [] : $discountCodes;

        $cartData = [
            'lines' => $lines,
            'note'=> $note,
            'attributes'=> $attributes,
            'discountCodes'=> $discountCodes,
        ];

        $cartData = $this->Storefront_Cart->GraphQL->normalize_buyer_identity($buyerIdentity, $cartData, true);

        $response = $this->create_cart($cartData, $buyerIdentity);

        if (\is_wp_error($response)) {
            return \wp_send_json_error($response);
        }

        return \wp_send_json_success($response);
    }

    public function add_shopwp_info_to_lines($lines = []) {

        if (empty($lines)) {
            return [];
        }

        return array_map(function($line) {

            $page = get_page_by_title($line->node->merchandise->product->title, 'OBJECT', 'wps_products');

            if (empty($page)) {
                $line->node->{"shopwp"} = false;

            } else {
                $product_id = get_post_meta($page->ID, 'product_id', true);

                $line->node->{"shopwp"} = [
                    'postId'    => $page->ID,
                    'productId' => $product_id,
                    'url'       => $page->guid
                ];
            }

            return $line;

        }, $lines);
    }

    

    public function handle_get_cart($request) {

        $cart_id            = $request->get_param('id');
        $buyerIdentity      = $request->get_param('buyerIdentity');
        
        if (empty($cart_id)) {
            return \wp_send_json_error(Utils::wp_error(__('Data for get cart callback is empty, exiting.', 'shopwp')));
        }

        $cartData = [
            'id'            => $cart_id,
            'buyerIdentity' => $buyerIdentity
        ];

        $cartData = $this->Storefront_Cart->GraphQL->normalize_buyer_identity($buyerIdentity, $cartData);

        $cart = $this->Storefront_Cart->api_get_cart($cartData);

        if (is_wp_error($cart)) {
            return wp_send_json_error($cart);
        }

        if (empty($cart)) {
            return wp_send_json_error(__('No cart data found. Try clearing your browser cache and reloading the page', 'shopwp'));
        }

        if (!empty($cart->lines->edges)) {
            $cart->lines->edges = $this->add_shopwp_info_to_lines($cart->lines->edges);
        }

        return \wp_send_json_success($cart);
    }    

    public function sanitize_lineitems($lines) {
        
        if (empty($lines)) {
            return [];
        }

        return array_map(function($line) {

            $quant_from_client = $line['quantity'];

            if (empty($quant_from_client) || $quant_from_client === 'null') {
                $quant_from_client = 1;

            } else {
                $quant_from_client = $line['quantity'];
            }

            $line_data = [
                'quantity' => $quant_from_client
            ];

            if (isset($line['variantId']) && !isset($line['merchandiseId'])) {
                $line_data['merchandiseId'] = \base64_decode($line['variantId']);

            } else {

                if (is_string($line['merchandiseId']) && Utils::str_contains($line['merchandiseId'], 'gid://shopify/ProductVariant/')) {
                    $line_data['merchandiseId'] = $line['merchandiseId'];
                } else {
                    $line_data['merchandiseId'] = 'gid://shopify/ProductVariant/' . $line['merchandiseId'];
                }
            }

            if (!empty($line['attributes'])) {
                $line_data['attributes'] = $line['attributes'];
            }

            if (!empty($line['sellingPlanId'])) {
                $line_data['sellingPlanId'] = 'gid://shopify/SellingPlan/' . $line['sellingPlanId'];
            }

            if (!empty($line['id'])) {
                $line_data['id'] = $line['id'];
            }

            return $line_data;

        }, $lines);
    }

    public function handle_add_lineitems($request) {
        
        $cartId             = $request->get_param('cartId');
        $lines              = $request->get_param('lines');
        $buyerIdentity      = $request->get_param('buyerIdentity');
        $metafields         = $request->get_param('metafields');

        if (empty($cartId)) {

            $response = $this->create_cart($cartData, $buyerIdentity);

            if (\is_wp_error($response)) {
                return \wp_send_json_error($response);
            }

            $cartId = $response->id;
            
        }

        $cartData = [
            'cartId' => $cartId,
            'lines' => $this->sanitize_lineitems($lines)
        ];

        if (!empty($metafields) ) {
            $cartData['metafields'] = $metafields;
        }

        $cartData = $this->Storefront_Cart->GraphQL->normalize_buyer_identity($buyerIdentity, $cartData);
        
        $response = $this->Storefront_Cart->api_add_lineitems($cartData);

        if (!$response) {
            return \wp_send_json_error(Utils::wp_error(__('Failed to add product to cart for unknown reason. Please try a different variant.', 'shopwp')));
        }

        if (\is_wp_error($response)) {
            return \wp_send_json_error($response);
        }

        return \wp_send_json_success($response);
    }

    public function handle_remove_lineitems($request) {

        $cartId             = $request->get_param('cartId');
        $lineIds            = $request->get_param('lineIds');
        $buyerIdentity      = $request->get_param('buyerIdentity');
        $metafields         = $request->get_param('metafields');

        if (empty($cartId)) {
            return \wp_send_json_error(Utils::wp_error(__('No cart id found during removing line items, exiting.', 'shopwp')));
        }

        $cartData = [
            'cartId' => $cartId,
            'lineIds' => !empty($lineIds) ? $lineIds : [],
            'metafields' => !empty($metafields) ? $metafields : false
        ];

        $cartData = $this->Storefront_Cart->GraphQL->normalize_buyer_identity($buyerIdentity, $cartData);

        $response = $this->Storefront_Cart->api_remove_lineitems($cartData);

        if (is_wp_error($response)) {
            return wp_send_json_error($response);
        }

        return \wp_send_json_success($response);
    }

    public function handle_update_lineitems($request) {

        $cartId             = $request->get_param('cartId');
        $lines              = $request->get_param('lines');
        $buyerIdentity      = $request->get_param('buyerIdentity');
        $metafields         = $request->get_param('metafields');

        if (empty($cartId)) {
            return \wp_send_json_error(Utils::wp_error(__('No cart id found during updating line items, exiting.', 'shopwp')));
        }

        $cartData = [
            'cartId'        => $cartId,
            'lines'         => !empty($lines) ? $this->sanitize_lineitems($lines) : [],
            'metafields'    => !empty($metafields) ? $metafields : false
        ];

        $cartData = $this->Storefront_Cart->GraphQL->normalize_buyer_identity($buyerIdentity, $cartData);

        $response = $this->Storefront_Cart->api_update_lineitems($cartData);

        if (is_wp_error($response)) {
            return wp_send_json_error($response);
        }

        return \wp_send_json_success($response);
    }

    public function handle_update_note($request) {

        $cartId             = $request->get_param('cartId');
        $buyerIdentity      = $request->get_param('buyerIdentity');
        $note               = $request->get_param('note');
        $metafields         = $request->get_param('metafields');

        if (empty($cartId)) {
            return \wp_send_json_error(Utils::wp_error(__('No cart id found during note update, exiting.', 'shopwp')));
        }

        $cartData = [
            'cartId' => $cartId,
            'note' => !empty($note) ? $note : '',
            'metafields' => !empty($metafields) ? $metafields : false
        ];

        $cartData = $this->Storefront_Cart->GraphQL->normalize_buyer_identity($buyerIdentity, $cartData);
        $response = $this->Storefront_Cart->api_update_note($cartData);

        if (is_wp_error($response)) {
            return wp_send_json_error($response);
        }

        return \wp_send_json_success($response);
        
    }    
    
    public function register_routes() {
        $this->api_route('/cart/create', 'POST', [$this, 'handle_create_cart']);
        $this->api_route('/cart/attributes/update', 'POST', [$this, 'handle_update_cart_attributes']);
        $this->api_route('/cart/discount', 'POST', [$this, 'handle_apply_discount']);
        $this->api_route('/cart/lineitems/update', 'POST', [$this, 'handle_update_lineitems']);
        $this->api_route('/cart/lineitems/add', 'POST', [$this, 'handle_add_lineitems']);
        $this->api_route('/cart/lineitems/remove', 'POST', [$this, 'handle_remove_lineitems']);
        $this->api_route('/cart/note/update', 'POST', [$this, 'handle_update_note']);
        $this->api_route('/cart/get', 'POST', [$this, 'handle_get_cart']);
        $this->api_route('/cart/buyer/update', 'POST', [$this, 'handle_update_buyer_identity']);
    }

    public function init()
    {
        add_action('rest_api_init', [$this, 'register_routes']);
    }

}