<?php

namespace ShopWP\API\Items;

use ShopWP\Messages;
use ShopWP\Utils;
use ShopWP\Utils\Sorting;
use ShopWP\Transients;
use ShopWP\Options;
use ShopWP\CPT;

if (!defined('ABSPATH')) {
    exit();
}

class Collections extends \ShopWP\API
{

    public $DB_Settings_General;
    public $DB_Settings_Syncing;
    public $DB_Settings_Connection;
    public $Shopify_API;
    public $Admin_API_Metafields;
    public $Storefront_Collections;

    public function __construct(
        $DB_Settings_General,
        $DB_Settings_Syncing,
        $DB_Settings_Connection,
        $Shopify_API,
        $Admin_API_Metafields,
        $Storefront_Collections
    ) {
        $this->DB_Settings_General = $DB_Settings_General;
        $this->DB_Settings_Syncing = $DB_Settings_Syncing;
        $this->DB_Settings_Connection = $DB_Settings_Connection;
        $this->Shopify_API = $Shopify_API;
        $this->Admin_API_Metafields = $Admin_API_Metafields;
        $this->Storefront_Collections = $Storefront_Collections;
    }

    public function custom_collections_meta_info()
    {
        return [
            'post_type' => SHOPWP_COLLECTIONS_POST_TYPE_SLUG,
            'increment_name' => 'custom_collections',
        ];
    }
    

   public function get_custom_collections_paginated($page_link = false, $limit = 250, $combined_custom_collections = []) {

        $response = $this->Shopify_API->get_custom_collections_per_page(
            $page_link,
            $limit
        );

        if (is_wp_error($response)) {
            return $response;
        }

        // No additional pages left
        if (!$response) {
            return $combined_custom_collections;
        }

        $response_body = $this->Shopify_API->sanitize_response(
            $response['body']
        );

        $new_custom_collections = $response_body->custom_collections;
        $new_custom_collections_count = count($new_custom_collections);

        $new_custom_collections = CPT::add_props_to_items(
            $new_custom_collections,
            $this->custom_collections_meta_info()
        );

        // Save the result in memory
        $combined_custom_collections = array_merge(
            $combined_custom_collections,
            $new_custom_collections
        );

        if (!$this->Shopify_API->has_pagination($response)) {
            return $combined_custom_collections;
        }

        $page_link = $this->Shopify_API->get_pagination_link($response);

        if (empty($page_link)) {
            return $combined_custom_collections;
        }        

        return $this->get_custom_collections_paginated(
            $page_link,
            $limit,
            $combined_custom_collections
        );
    }

    public function cache_metafields($new_metafield_data) {
        
        $active_metafields = Options::get('shopwp_active_metafields');

        if (!empty($active_metafields)) {
            $active_metafields = maybe_unserialize($active_metafields);

        } else {
            $active_metafields = [];
        }

        $active_metafields[] = $new_metafield_data;

        return Options::update('shopwp_active_metafields', maybe_serialize($active_metafields));
    }

    public function metafields_compare($a, $b) {

        if ($a['id'] === $b['id']) {
            return 0;

        } else {
            return -1;
        }
         
    }

    public function uncache_metafields($metafield_id) {
        
        $active_metafields = Options::get('shopwp_active_metafields');

        if (!empty($active_metafields)) {
            $active_metafields = maybe_unserialize($active_metafields);

        } else {
            return false;
        }

        $new_metafields = array_filter($active_metafields, function($metafield) use($metafield_id) {
            return $metafield['id'] === $metafield_id;
        });

        $new_stuff = array_udiff($active_metafields, $new_metafields, [$this, 'metafields_compare']);

        return Options::update('shopwp_active_metafields', maybe_serialize($new_stuff));
    }

    public function handle_show_metafield($request) {

        $key = $request->get_param('key');
        $namespace = $request->get_param('namespace');

        $response = $this->Admin_API_Metafields->api_show_metafield($key, $namespace);

        if (is_wp_error($response)) {
            return wp_send_json_error($response);
        }

        $metafield_id = $response->id;

        $new_metafield_data = [
            'id' => $metafield_id,
            'key' => $key,
            'namespace' => $namespace
        ];

        $saved_metafields_result = $this->cache_metafields($new_metafield_data);
            
        return \wp_send_json_success($metafield_id);
    }

    public function handle_hide_metafield($request) {

        $metafield_id = $request->get_param('id');
        
        $hide_result = $this->Admin_API_Metafields->api_hide_metafield($metafield_id);
        
        $this->uncache_metafields($metafield_id);
            
        return $hide_result;
    }

     public function handle_get_collections($request) {

        $data = $request->get_param('data');
        $result = $this->get_collections($data);

        if (is_wp_error($result)) {
            return $this->handle_response($result);
        }

        return \wp_send_json_success($result);

    }

    /*
    
    Public API Method
    
    */
    public function get_collections($query_params) {

        $cached_enabled = $this->DB_Settings_General->col('enable_data_cache', 'bool');
        $data_cache_length = $this->DB_Settings_General->col('data_cache_length', 'int');

        if (empty($query_params)) {
            return \wp_send_json_error(Utils::wp_error(__('No data was found for get products, exiting.', 'shopwp')));
        }

        if (isset($query_params['withProducts'])) {
            $with_products = $query_params['withProducts'];
        } else {
            $with_products = false;
        }

        if (isset($query_params['queryParams'])) {
            $query_params = $query_params['queryParams'];
        }        

        if ($cached_enabled) {

            $hash = Utils::hash($query_params, true);

            $cached_query = Transients::get('shopwp_query_' . $hash);

            $cached_query = \maybe_unserialize($cached_query);

            if (!empty($cached_query)) {
                return $cached_query;
            }
        }

        $final_params = $this->public_api_default_values($query_params);

        $result = $this->Storefront_Collections->api_get_collections($final_params, false, $with_products);

        if (is_wp_error($result)) {
            return $result;
        }

        if ($cached_enabled) {
            Transients::set('shopwp_query_' . $hash, \maybe_serialize($result), $data_cache_length);
        }

        return $result;

    }

    /*

	Gets all collections

	*/
    public function handle_get_all_collections($request)
    {

        $cached_enabled         = $this->DB_Settings_General->col('enable_data_cache', 'bool');
        $data_cache_length      = $this->DB_Settings_General->col('data_cache_length', 'int');
        $all_collections        = Transients::get('shopwp_all_collections');

        if (!empty($all_collections)) {
            return $this->handle_response([
                'response' => $all_collections,
            ]);
        }

        $limit = $this->DB_Settings_General->get_items_per_request();
        $has_connection = $this->DB_Settings_Connection->has_connection();

        if (!$has_connection) {
            return wp_send_json_error(Messages::get('connection_not_found') . ' (handle_get_all_collections)');
        }

        $smart_collections_response = $this->Shopify_API->get_smart_collections_paginated(false, $limit, []);

        if (is_wp_error($smart_collections_response)) {
            return $this->handle_response([
                'response' => $smart_collections_response,
            ]);
        }

        $custom_collections_response = $this->get_custom_collections_paginated(false, $limit, []);

        if (is_wp_error($custom_collections_response)) {
            return $this->handle_response([
                'response' => $custom_collections_response,
            ]);
        }

        $smart_collections = $smart_collections_response;
        $custom_collections = $custom_collections_response;

        if (Utils::has($smart_collections, 'errors')) {
            return $this->handle_response([
                'response' => $smart_collections,
            ]);
        }

        if (Utils::has($custom_collections, 'errors')) {
            return $this->handle_response([
                'response' => $custom_collections,
            ]);
        }

        $collections_merged = array_merge(
            $smart_collections,
            $custom_collections
        );

        if (!empty($collections_merged)) {

            $collections_merged_final_reduced = array_map(function (
                $collection
            ) {
                $new_collection_obj = new \stdClass();
                $new_collection_obj->id = $collection->id;
                $new_collection_obj->title = $collection->title;
                $new_collection_obj->handle = $collection->handle;

               if (!empty($collection->rules)) {
                  $new_collection_obj->rules = true;
               }

                return $new_collection_obj;
            },
            $collections_merged);
            
            usort($collections_merged_final_reduced, function ($a, $b) {
               return strcmp($a->title, $b->title);
            });

            $serialized_collections = maybe_serialize($collections_merged_final_reduced);

            Transients::set('shopwp_all_collections', $serialized_collections, $data_cache_length);

            if (!$request) {
                return $collections_merged_final_reduced;
            }

            return \wp_send_json_success($collections_merged_final_reduced);

        } else {

            if (!$request) {
                return Utils::wp_error(__('No collections found!', 'shopwp'));
            }

            return wp_send_json_error('No collections found!');
        }
    }

    public function register_routes() {
        $this->api_route('/metafields/hide', 'POST', [$this, 'handle_hide_metafield']);
        $this->api_route('/metafields/show', 'POST', [$this, 'handle_show_metafield']);
        $this->api_route('/collections', 'POST', [$this, 'handle_get_collections']);
        $this->api_route('/collections/all', 'GET', [$this, 'handle_get_all_collections']);
    }

    public function init()
    {
        add_action('rest_api_init', [$this, 'register_routes']);
    }
}
