<?php

namespace ShopWP;

if (!defined('ABSPATH')) {
    exit();
}

use ShopWP\Utils;
use ShopWP\Utils\Data as Utils_Data;
use ShopWP\Utils\Server;

class Media_Uploader
{
    protected $DB_Settings_Syncing;
    protected $DB_Images;

    public function __construct($DB_Settings_Syncing, $DB_Images)
    {
        $this->DB_Settings_Syncing = $DB_Settings_Syncing;
        $this->DB_Images = $DB_Images;
    }

    public function upload_image($image, $existing_attachments, $post_id, $is_featured = true) {

        $process_media_results = $this->process_media(
            $image,
            $existing_attachments,
            $post_id,
            $is_featured
        );

        \wp_cache_delete('shopwp_data_sync_existing_attachments');

        return $process_media_results;
    }

    public function media_file_already_exists($filename) {

        global $wpdb;
        
        $query = "SELECT meta_value FROM {$wpdb->postmeta} WHERE meta_key = '_wp_attachment_metadata' AND meta_value LIKE '%$filename%'";

        $row = $wpdb->get_row($query);

        if ($row && !empty($row->meta_value)) {

            $meta_value = \maybe_unserialize($row->meta_value);
            
            if (empty($meta_value['attachment_id'])) {
                $upload_dir                     = \wp_upload_dir();
                $file_url                       = $upload_dir['baseurl'] . '/' . $meta_value['file'];
                $new_attachment_id              = \attachment_url_to_postid($file_url);    
                $meta_value['attachment_id']    = $new_attachment_id;
            }

            return $meta_value;
        
        }
            
        return false;

    }

    public function process_media($media_item, $existing_attachments, $post_id, $is_featured = false)
    {

        if (property_exists($media_item, 'url')) {
            $file_info          = pathinfo($media_item->url);
            $img_url            = $media_item->url;

        } else if (property_exists($media_item, 'image')) {
            $file_info          = pathinfo($media_item->image->url);
            $img_url            = $media_item->image->url;
            $media_item->url    = $media_item->image->url;

        } else {
            $this->DB_Settings_Syncing->log(__('Couldn\'t find the correct pathinfo from image URL. Skipping image ...', 'shopwp'), 'warning');
            return false;
        }

        $file_name = $file_info['basename'];
        $file_name_without_params = explode('?', $file_name)[0];
        $media_already_exists = $this->media_file_already_exists($file_name_without_params);

        if ($is_featured && $file_name) {

            if ($media_already_exists && $post_id) {

                $this->DB_Settings_Syncing->log(__('Found existing image, setting featured image and moving on ...', 'shopwp'));

                if (!empty($media_already_exists['attachment_id'])) {

                    // Should return false
                    $set_feat_result = \set_post_thumbnail(
                        $post_id,
                        $media_already_exists['attachment_id']
                    );

                    Utils::log([
                        'thing_to_log'  => '--- ATTACHMENT ID IS NOT EMPTY ---',
                        'log_level'     => 'info',
                        'file'          => 'shopwp-debug'
                    ]);

                    return [
                        'src' => $img_url,
                        'attachment_id' => empty($media_already_exists['attachment_id']) ? false : $media_already_exists['attachment_id'],
                    ];

                } else {

                    $this->DB_Settings_Syncing->log(__('Post ID found but no existing attachment ID found.', 'shopwp'), 'info');
                    $set_feat_result = false;

                    Utils::log([
                        'thing_to_log'  => '--- ATTACHMENT ID IS EMPTY ---',
                        'log_level'     => 'info',
                        'file'          => 'shopwp-debug'
                    ]);

                }

            }

        }

        if (empty($media_already_exists)) {

            $this->DB_Settings_Syncing->log(__('Downloading product image ', 'shopwp') . $img_url . ' ... ');

            $file_info = $this->download_media_file(
                $media_item,
                $existing_attachments
            );
            
            if (empty($file_info)) {
                return false;
            }

            if (empty($file_info['basename'])) {
                $name = $file_info['orig_src'];

            } else {
                $name = $file_info['basename'];
            }

            $this->DB_Settings_Syncing->log(__('Uploading the image ... ', 'shopwp'));

            $result = $this->upload_media_file($file_info, $post_id);

            return $result;

        } else {
            return true;
        }

    }

    public function upload_media_file($file, $post_id)
    {
        if (is_wp_error($file)) {
            $this->DB_Settings_Syncing->log($file->get_error_message(), 'error');
            return $file;
        }

        if ($file['existing']) {
            
            $set_feat_result = \set_post_thumbnail(
                $post_id,
                $file['attachment_id']
            );

            return [
                'src' => $file['orig_src'],
                'attachment_id' => $file['attachment_id'],
            ];
        }

        $file_array = [
            'name' => $file['basename'], //isolates and outputs the file name from its absolute path
            'type' => $file['mime'], //yes, thats sloppy, see my text further down on this topic
            'tmp_name' => $file['image'], //this field passes the actual path to the image
            'error' => 0,
            'size' => $file['size'], //returns image filesize in bytes
        ];

        $file_attributes = \wp_handle_sideload(
            $file_array,
            $this->sideload_overrides()
        );

        if (!empty($file_attributes['error'])) {

            $this->DB_Settings_Syncing->log($file_attributes['error'], 'warning');

            return Utils::wp_error($file_attributes['error']);
        }

        $attachment = [
            'guid' => $file_attributes['url'],
            'post_mime_type' => $file['mime'],
            'post_title' => \get_the_title($post_id),
            'post_status' => 'inherit',
        ];

        $attachment_id = \wp_insert_attachment(
            $attachment,
            $file_attributes['file'],
            $post_id,
            true
        );
        
        if (is_wp_error($attachment_id)) {
            $this->DB_Settings_Syncing->log($attachment_id->get_error_message(), 'warning');
            return $attachment_id;
        }

        if ( !function_exists('wp_generate_attachment_metadata')) {
            include( ABSPATH . 'wp-admin/includes/image.php' );
        }

        // Generate the metadata for the attachment, and update the database record.
        $attach_data = \wp_generate_attachment_metadata(
            $attachment_id,
            $file_attributes['file']
        );

        // Important
        $attach_data['shopwp-media'] = true;
        $attach_data['orig_src'] = $file['orig_src'];
        $attach_data['attachment_id'] = $attachment_id;

        $update_meta_result = \wp_update_attachment_metadata(
            $attachment_id,
            $attach_data
        );

        $media_post_updated = \wp_update_post([
            'ID'            => $attachment_id,
            'post_parent'   => $post_id,
        ], true);

        if ($update_meta_result === false) {
            $this->DB_Settings_Syncing->log(__('Unable to update attachment metadata for attachment id: ', 'shopwp') . $attachment_id . __('. Post is invalid.', 'shopwp'), 'warning');
        }

        $update_alt_result = \update_post_meta(
            $attachment_id,
            '_wp_attachment_image_alt',
            $file['alt']
        );

        if ($post_id) {

            $set_feat_result = \set_post_thumbnail(
                $post_id,
                $attachment_id
            );

            if ($set_feat_result === false) {
                $this->DB_Settings_Syncing->log(__('Unable to set post thumbnail for attachment id: ', 'shopwp') . $attachment_id, 'warning');
            }
        }

        return [
            'src' => $file['orig_src'],
            'attachment_id' => $attachment_id,
        ];
    }

    /*
   
   Renames the temp download file
   
   */
    public function rename_temp_file($temp_file_path, $mime_and_extension)
    {
        if (empty($temp_file_path)) {
            return false;
        }

        $exploded = explode('.tmp', $temp_file_path);

        return $exploded[0] . $mime_and_extension['ext'];
    }

    public function attachment_exists($existing_attachments, $image)
    {
        if (!is_array($existing_attachments)) {
            return [];
        }

        if (empty($image)) {
            return [];
        }

        return array_filter($existing_attachments, function (
            $existing_attachment
        ) use ($image) {
            return Utils::str_contains($existing_attachment->meta_value, $image->url);
        });
    }

    public function get_attachment_id_from_existing($existing_attachment)
    {
        $reset_indexes = array_values($existing_attachment);

        $existing_attachments = maybe_unserialize(
            $reset_indexes[0]->meta_value
        );

        if (!empty($existing_attachments)) {
            return $existing_attachments['attachment_id'];
        }

        return false;
    }

    public function download_multiple_media_files(
        $images,
        $existing_attachments
    ) {
        $results = [];

        foreach ($images as $image) {
            $results[] = $this->download_media_file(
                $image,
                $existing_attachments
            );
        }

        return $results;
    }

    public function build_download_results(
        $temp_file_path,
        $temp_file_path_mod,
        $mime_and_extension,
        $image
    ) {
        return [
            'image' => $temp_file_path_mod,
            'mime' => $mime_and_extension['type'],
            'post_id' => isset($image->postId) ? $image->postId : false,
            'basename' => basename($temp_file_path_mod),
            'size' => filesize($temp_file_path),
            'ext' => $mime_and_extension['ext'],
            'alt' => $image->altText,
            'orig_src' => $image->url,
            'existing' => false,
        ];
    }

    

    /*

    This tells WordPress to not look for the POST form
    fields that would normally be present. Default is true.
    Since the file is being downloaded from a remote server,
    there will be no form fields.

    */
    public function sideload_overrides()
    {
        return [
            'test_form'     => false,
            'test_upload'   => false,
            'test_type'     => false,
            'test_size'     => true,
        ];
    }


    /*

    Finds the required file info

    */
    public function get_file_info($file, $url)
    {
        if (function_exists('mime_content_type')) {
            $mime_type = \mime_content_type($file);

            if ($mime_type === 'image/png') {
                $ext = '.png';
            } elseif (
                $mime_type === 'image/jpeg' ||
                $mime_type === 'image/pjpeg'
            ) {
                $ext = '.jpg';
            } elseif ($mime_type === 'image/gif') {
                $ext = '.gif';
            }

            return [
                'type' => $mime_type,
                'ext' => $ext,
            ];

        } else {
            
            /*
			  
			Find the extension and mime type manually if native PHP function
            mime_content_type is not available...
			  
			*/
			$split_parts    = \explode("cdn.shopify.com", $url);
			$next_part      = \explode(".", $split_parts[1]);
			$final_part     = \explode("?", $next_part[1]);

            if (empty($final_part)) {
                return false;
            }

			$ext = '.' . $final_part[0];

			if ($ext === '.gif') {
				$mime_type = 'image/gif';

			} else if ($ext === '.jpeg' || $ext === '.jpg') {
				$mime_type = 'image/jpeg';

			} else if ($ext === '.png') {
				$mime_type = 'image/png';

			} else if ($ext === '.svg') {
				$mime_type = 'image/svg+xml';

			} else if ($ext === '.tiff') {
				$mime_type = 'image/tiff';

			} else if ($ext === '.webp') {
				$mime_type = 'image/webp';
			} else {
                return false;
            }

			return [
				'type' => $mime_type,
				'ext' => $ext,
			];
        }
    }


    /*
   
   Downloads a chunk of media
   
   */
    public function download_media_file($image, $existing_attachments)
    {
        $result = false;

        if (empty($image) || empty($image->url)) {
            $this->DB_Settings_Syncing->log(__('Product image is empty, skipping ... ', 'shopwp'), 'warning');
            return $result;
        }

        $existing_attachment = $this->attachment_exists(
            $existing_attachments,
            $image
        );

        if (!empty($existing_attachment)) {
            $existing_attachment_id = $this->get_attachment_id_from_existing(
                $existing_attachment
            );

            if ($existing_attachment_id) {
                $result = [
                    'existing' => true,
                    'post_id' => isset($image->postId) ? $image->postId : false,
                    'attachment_id' => $existing_attachment_id,
                    'orig_src' => $image->url,
                ];

                // Updates the front-end file status
                $this->DB_Settings_Syncing->update_recently_syncd_media_ref(
                    $image->url
                );

                // continue;
                return $result;
            }
        }

        /*

        Downloads image to a local temporary file using the WordPress HTTP API.

        */
        $temp_file_path = \download_url($image->url);

        if (\is_wp_error($temp_file_path)) {
            $result = $temp_file_path;

            $download_error = $temp_file_path->get_error_message();

            @unlink($temp_file_path);

            // continue;
            return $result;
        }

        if (empty($temp_file_path)) {
            $this->DB_Settings_Syncing->log(__('Product image empty after downloading, skipping ... ', 'shopwp'), 'warning');
            return false;
        }

        $mime_and_extension = $this->get_file_info($temp_file_path, $image->url);

        if ($mime_and_extension === false) {
            $this->DB_Settings_Syncing->log(__('Unable to find mime type or extension of file ... ', 'shopwp'), 'warning');
            return false;
        }

        $temp_file_path_mod = $this->rename_temp_file(
            $temp_file_path,
            $mime_and_extension
        );

        $result = $this->build_download_results(
            $temp_file_path,
            $temp_file_path_mod,
            $mime_and_extension,
            $image
        );

        if ($temp_file_path_mod) {
            rename($temp_file_path, $temp_file_path_mod);
        }

        $this->DB_Settings_Syncing->update_recently_syncd_media_ref(
            $image->url
        );

        if ($temp_file_path) {
            @unlink($temp_file_path);
        }

        return $result;
    }

    
}
