<?php

namespace ShopWP;

use ShopWP\Messages;
use ShopWP\Utils\Data as Utils_Data;

if (!defined('ABSPATH')) {
    exit();
}

class Utils
{
    public static function filter_errors($item)
    {
        return is_wp_error($item);
    }

    public static function filter_error_messages($error)
    {
        if (isset($error->errors) && isset($error->errors['error'])) {
            return $error->errors['error'][0];
        }
    }

    public static function wp_error($params)
    {
        if (is_string($params)) {
            return new \WP_Error('error', $params);
        }

        return new \WP_Error('error', Messages::error($params));
    }

    public static function return_only_errors($items)
    {
        return array_filter(
            $items,
            [__CLASS__, 'filter_errors'],
            ARRAY_FILTER_USE_BOTH
        );
    }

    public static function return_only_error_messages($array_of_errors)
    {
        return array_values(
            array_map([__CLASS__, 'filter_error_messages'], $array_of_errors)
        );
    }

    public static function hash_rand($content, $serialize = false)
    {
        $bytes = \random_int(1, 200);

        if ($serialize) {
            return md5(serialize($content)) . wp_rand(5, 8) . $bytes;
        }

        return md5($content) . wp_rand(5, 8) . $bytes;
    }

    public static function hash($content, $serialize = false)
    {
        if ($serialize) {
            return md5(serialize($content));
        }

        return md5($content);
    }

    public static function base64($content, $type = 'encode')
    {
        if ($type === 'encode') {
            return base64_encode(json_encode($content));
        }

        return json_decode(base64_decode($content));
    }

    public static function hash_static_num($content)
    {
        return crc32(self::hash_rand($content));
    }

    public static function get_abs_admin_path($path = '')
    {
        return rtrim(ABSPATH, '/') . '/wp-admin/' . $path;
    }

    public static function is_plugin_installed($plugin_basename)
    {
        $installed_plugins = \get_plugins();

        return array_key_exists($plugin_basename, $installed_plugins);
    }

    public static function convert_to_comma_string($items)
    {
        if (is_string($items)) {
            return $items;
        }

        if (is_array($items)) {
            return implode(', ', $items);
        }

        return false;
    }

    public static function is_multi_array($array)
    {
        if (empty($array)) {
            return false;
        }
        $rest = reset($array);

        return is_array($rest);
    }

    public static function get_first_image_if_exists($product)
    {
        if (self::has($product, 'images') && !empty($product->images)) {
            return $product->images[0]->src;
        }
    }

    public static function flatten_image_prop($items)
    {
        $items_copy = $items;
        $items_copy = self::convert_array_to_object($items_copy);

        if (
            self::has($items_copy, 'image') &&
            self::has($items_copy->image, 'src')
        ) {
            $items_copy->image = $items_copy->image->src;
        } else {
            $items_copy->image = self::get_first_image_if_exists($items_copy);
        }

        return $items_copy;
    }

    public static function filter_items_by_id($items, $diff, $keyToCheck = 'id')
    {
        $result = [];

        foreach ($items as $key => $value) {
            foreach ($diff as $key => $diffID) {
                if (is_object($value)) {
                    if ($diffID === $value->$keyToCheck) {
                        $result[] = $value;
                    }
                } else {
                    if ($diffID === $value[$keyToCheck]) {
                        $result[] = $value;
                    }
                }
            }
        }

        return $result;
    }

    public static function gather_item_ids(
        $current_items,
        $new_items,
        $num_dimensions,
        $key_to_check
    ) {
        return [
            'current' => self::get_item_ids(
                $current_items,
                $num_dimensions,
                $key_to_check
            ),
            'new' => self::get_item_ids(
                $new_items,
                $num_dimensions,
                $key_to_check
            ),
        ];
    }

    public static function find_items_to_delete(
        $current_items,
        $new_items,
        $num_dimensions = false,
        $key_to_check = 'id'
    ) {
        $ids_to_check = self::gather_item_ids(
            $current_items,
            $new_items,
            $num_dimensions,
            $key_to_check
        );

        // Deletes ids in 'current' that arent in 'new'
        $difference = array_values(
            array_diff($ids_to_check['current'], $ids_to_check['new'])
        );

        return self::filter_items_by_id(
            $current_items,
            $difference,
            $key_to_check
        );
    }

    public static function find_items_to_add(
        $current_items,
        $new_items,
        $num_dimensions = false,
        $key_to_check = 'id'
    ) {
        $ids_to_check = self::gather_item_ids(
            $current_items,
            $new_items,
            $num_dimensions,
            $key_to_check
        );

        // Adds ids from 'new' that arent in 'current'
        $difference = array_values(
            array_diff($ids_to_check['new'], $ids_to_check['current'])
        );

        return self::filter_items_by_id($new_items, $difference, $key_to_check);
    }

    public static function get_item_ids(
        $items,
        $one_dimension = false,
        $key_to_check = 'id'
    ) {
        $items = self::convert_to_assoc_array($items);

        $results = [];

        if ($one_dimension) {
            foreach ($items as $item) {
                if (isset($item[$key_to_check]) && $item[$key_to_check]) {
                    $results[] = $item[$key_to_check];
                }
            }
        } else {
            foreach ($items as $sub_array) {
                foreach ($sub_array as $item) {
                    if (isset($item[$key_to_check]) && $item[$key_to_check]) {
                        $results[] = $item[$key_to_check];
                    }
                }
            }
        }

        return $results;
    }

    public static function convert_object_to_array($maybe_object)
    {
        if (is_array($maybe_object)) {
            return $maybe_object;
        }

        if (
            is_float($maybe_object) ||
            is_int($maybe_object) ||
            is_bool($maybe_object)
        ) {
            return self::wp_error([
                'message_lookup' => 'unable_to_convert_to_array',
                'call_method' => __METHOD__,
                'call_line' => __LINE__,
            ]);
        }

        return (array) $maybe_object;
    }

    public static function convert_array_to_object($maybe_array)
    {
        if (is_object($maybe_array)) {
            return $maybe_array;
        }

        // Unable to convert to Object from these. Return false.
        if (
            is_float($maybe_array) ||
            is_int($maybe_array) ||
            is_bool($maybe_array)
        ) {
            return self::wp_error([
                'message_lookup' => 'unable_to_convert_to_object',
                'call_method' => __METHOD__,
                'call_line' => __LINE__,
            ]);
        }

        if (is_array($maybe_array)) {
            return json_decode(json_encode($maybe_array), false);
        }
    }

    public static function convert_to_assoc_array($items)
    {
        return json_decode(json_encode($items), true);
    }

    public static function serialize_data_for_db($data)
    {
        $dataSerialized = [];

        foreach ($data as $key => $value) {
            /*

			IMPORTANT -- Need to check for both Array and Objects
			otherwise the following error is thrown and data not saved:

			mysqli_real_escape_string() expects parameter 2 to be string, object given

			*/
            if (is_array($value) || is_object($value)) {
                $value = maybe_serialize($value);
            }

            $dataSerialized[$key] = $value;
        }

        return $dataSerialized;
    }

    public static function data_values_size_limit_reached($data, $table_name)
    {
        global $wpdb;

        foreach ($data as $key => $value) {
            $db_col_size = $wpdb->get_col_length($table_name, $key);

            if ($db_col_size !== false && !is_wp_error($db_col_size)) {
                if (
                    Utils_Data::size_in_bytes($value) > $db_col_size['length']
                ) {
                    return [
                        'table_name' => $table_name,
                        'value_attempted' => $value,
                        'column_name' => $key,
                        'max_size' => $db_col_size['length'],
                    ];
                }
            }
        }

        return false;
    }

    public static function mask($string)
    {
        $length = strlen($string);
        $stringNew =
            str_repeat('•', $length - 4) .
            $string[$length - 4] .
            $string[$length - 3] .
            $string[$length - 2] .
            $string[$length - 1];
        return $stringNew;
    }

    public static function remove_spaces_from_string($string)
    {
        return str_replace(' ', '', $string);
    }

    public static function comma_list_to_array($string)
    {
        return array_map('trim', explode(',', $string));
    }

    public static function prevent_timeouts()
    {
        if (!function_exists('ini_get') || !ini_get('safe_mode')) {
            @set_time_limit(0);
        }
    }

    public static function has($item, $property)
    {
        if (is_array($item)) {
            $item = self::convert_array_to_object($item);
        }

        return is_object($item) && property_exists($item, $property)
            ? true
            : false;
    }

    public static function array_not_empty($maybe_array)
    {
        if (is_array($maybe_array) && !empty($maybe_array)) {
            return true;
        } else {
            return false;
        }
    }

    public static function object_is_empty($object)
    {
        $object_copy = $object;
        $object_copy = (array) $object_copy;

        if (count(array_filter($object_copy)) == 0) {
            return true;
        } else {
            return false;
        }
    }

    public static function maybe_wrap_in_array($something)
    {
        if (!is_array($something)) {
            $something = [$something];
        }

        return $something;
    }

    public static function convert_to_https_url($url)
    {
        if (self::str_contains_start($url, '://')) {
            return 'https://' . explode('://', $url)[1];
        }

        return $url;
    }

    public static function unset_by($object, $keys = [])
    {
        foreach ($keys as $key) {
            unset($object->{$key});
        }

        return $object;
    }

    public static function unset_all_except($item, $exception)
    {
        if (!self::has($item, $exception)) {
            return $item;
        }

        foreach ($item as $key => $value) {
            if ($key !== $exception) {
                unset($item->{$key});
            }
        }

        return $item;
    }

    public static function filter_data_by($items, $criteria = [])
    {
        if (!$criteria) {
            return $items;
        }

        if (is_object($items)) {
            $items = self::convert_object_to_array($items);
        }

        return array_map(function ($item) use ($criteria) {
            return self::unset_by($item, $criteria);
        }, $items);
    }

    public static function filter_data_except($items, $exception = false)
    {
        if (!$exception) {
            return $items;
        }

        if (is_object($items)) {
            $items = self::convert_object_to_array($items);
        }

        return array_map(function ($item) use ($exception) {
            return self::unset_all_except($item, $exception);
        }, $items);
    }

    public static function new_cols_greater_than_old(
        $columns_new,
        $columns_current
    ) {
        return count($columns_new) > count($columns_current);
    }

    public static function new_cols_less_than_old(
        $columns_new,
        $columns_current
    ) {
        return count($columns_new) < count($columns_current);
    }

    public static function flatten_array($array)
    {
        $result = [];

        if (!is_array($array)) {
            $array_new = func_get_args();
        } else {
            $array_new = $array;
        }

        foreach ($array_new as $key => $value) {
            if (is_array($value)) {
                $result = array_merge($result, self::flatten_array($value));
            } else {
                $result = array_merge($result, [$key => $value]);
            }
        }

        return $result;
    }

    public static function first_num($num)
    {
        $num_split = str_split($num);

        return (int) $num_split[0];
    }

    public static function is_network_wide()
    {
        if (!function_exists('is_plugin_active_for_network')) {
            require_once self::get_abs_admin_path('includes/plugin.php');
        }

        return \is_plugin_active_for_network(SHOPWP_BASENAME);
    }

    public static function get_site_url($blog_id = false)
    {
        if (is_multisite()) {
            if ($blog_id) {
                $blog_details = get_blog_details($blog_id);
            } else {
                $blog_details = get_blog_details(get_current_blog_id());
            }

            if (!empty($blog_details)) {
                return $blog_details->home;
            }
        } else {
            return get_home_url();
        }
    }

    public static function get_string_between($string, $start, $end)
    {
        $string = ' ' . $string;
        $ini = strpos($string, $start);
        if ($ini == 0) {
            return '';
        }
        $ini += strlen($start);
        $len = strpos($string, $end, $ini) - $ini;
        return substr($string, $ini, $len);
    }

    public static function str_contains($haystack, $needle)
    {
        if (!is_string($haystack) || !is_string($needle)) {
            return false;
        }

        if (strpos($haystack, $needle) !== false) {
            return true;
        }

        return false;
    }

    public static function str_contains_start($string, $needle)
    {
        if (!is_string($string) || !is_string($needle)) {
            return false;
        }

        if (strpos($string, $needle) === 0) {
            return true;
        }

        return false;
    }

    public static function convert_camel_to_underscore($string)
    {
        return strtolower(preg_replace('/(?<=\\w)(?=[A-Z])/', '_$1', $string));
    }

    public static function sanitize($key, $value)
    {
        if ($key === 'cart_terms_content') {
            return self::setting_to_terms($value);
        } elseif ($key === 'sync_by_collections' || $key === 'javascript_overrides') {
            return \maybe_serialize($value);

        } elseif ($key === 'sync_by_webhooks') {

            if (empty($value)) {
                return '';
            }

            return \maybe_serialize($value);
        }

        return Utils_Data::sanitize_setting($value, true);
    }

     public static function setting_to_terms($value)
    {
        return wp_kses($value, [
            'strong' => [],
            'b' => [],
            'i' => [],
            'em' => [],
            'a' => [
                'href' => [],
                'title' => [],
                'target' => [],
            ],
        ]);
    }

    public static function convert_settings_shape($settings, $also_sanitize = false)
    {
        $new_settings = [];

        if (empty($settings)) {
            return $new_settings;
        }

        foreach ($settings as $setting_key => $setting_val) {

            // Set new key
            $key = self::convert_camel_to_underscore($setting_key);

            if ($also_sanitize) {
                $setting_val = self::sanitize($key, $setting_val);
            }

            $new_settings[$key] = $setting_val;
        }

        return $new_settings;

    }

    public static function convert_underscore_to_camel_string($string)
    {
        return lcfirst(str_replace(' ', '', ucwords(str_replace('_', ' ', $string))));;
    }

    public static function convert_hyphen_to_underscore($string)
    {
        return str_replace('-', '_', $string);
    }

    public static function convert_space_to_underscore($string)
    {
        return str_replace(' ', '_', $string);
    }

    public static function convert_slash_to_underscore($string)
    {
        return strtolower(preg_replace('/[\W\s\/]+/', '_', $string));
    }

    public static function convert_underscore_to_camel_array($array, $exceptions = [])
    {
        $arr = [];

        foreach ($array as $key => $value) {

            if (!in_array($key, $exceptions)) {
                $key = self::convert_underscore_to_camel_string($key);

                if (is_array($value) || is_object($value)) {
                    $value = self::convert_underscore_to_camel_array($value, $exceptions);
                }
            }

            $arr[$key] = $value;
        }

        return $arr;
    }

    public static function mask_value($val)
    {
        $length_str = strlen($val);

        if ($length_str <= 4) {
            return $val;
        }

        $str_length_minus_four = $length_str - 4;
        $str_last_four_chars = substr($val, -4);

        return str_repeat("*", $str_length_minus_four) . $str_last_four_chars;
    }

    public static function strip_tags_content(
        $text,
        $tags = '',
        $invert = false
    ) {

        if (empty($tags)) {
            return $text;
        }

        preg_match_all('/<(.+?)[\s]*\/?[\s]*>/si', trim($tags), $tags);

        $tags = array_unique($tags[1]);

        if (is_array($tags) and count($tags) > 0) {
            if ($invert == false) {
                return preg_replace(
                    '@<(?!(?:' .
                        implode('|', $tags) .
                        ')\b)(\w+)\b.*?>.*?</\1>@si',
                    '',
                    $text
                );
            } else {
                return preg_replace(
                    '@<(' . implode('|', $tags) . ')\b.*?>.*?</\1>@si',
                    '',
                    $text
                );
            }
        } elseif ($invert == false) {
            return preg_replace('@<(\w+)\b.*?>.*?</\1>@si', '', $text);
        }

        return $text;
    }

    public static function sanitize_html_template($html)
    {
        $final_html_template_data = preg_replace('~>\\s+<~m', '><', $html);

        return base64_encode($final_html_template_data);
    }

    public static function convert_storefront_id_to_numeric($storefront_id, $type = 'Product') {

        if (strpos($storefront_id, 'gid://') === false && $type === 'Product') {
            $id_to_check = base64_decode($storefront_id);
        } else {
            $id_to_check = $storefront_id;
        }

        $result = str_replace('gid://shopify/' . $type . '/', '', $id_to_check);

        return (int) $result;        
    }    

    public static function log($params = ['thing_to_log' => '', 'log_level' => 'info', 'file' => 'shopwp']) { 

        $mode           = 'a';
        $file           = empty($params['file']) ? 'shopwp' : $params['file'];
        $thing_to_log   = $params['thing_to_log'];
        $log_level      = $params['log_level'];
        
        $upload_dir     = \wp_upload_dir();
        $upload_dir     = $upload_dir['basedir'];
        
        // If the thing_to_log is array, json_encode.
        if ( is_array( $thing_to_log ) || is_object( $thing_to_log ) ) { 
            $thing_to_log = json_encode( $thing_to_log ); 
        } 

        // Write the log file.
        $file  = $upload_dir . '/' . $file . '.log';
        $file  = fopen( $file, $mode );

        $datetime = \wp_date('g:i A');

        if ($log_level === 'info') {
            $preline = 'Info';

        } else if ($log_level === 'warning') {
            $preline = 'Warning';

        } else if ($log_level === 'error') {
            $preline = 'Error';
        }
        
        $bytes = fwrite( $file, $datetime . ' :: ' . $preline . ' :: ' . $thing_to_log . PHP_EOL); 
        
        fclose( $file ); 

        return $bytes;

    }

    public static function create_numeric_id($full_gid, $type = 'Product') {
        return str_replace('gid://shopify/' . $type .'/', '', $full_gid);
    }

    public static function exceeds_max_post_body_size($data)
    {
        return Utils_Data::size_in_bytes($data) > \wp_max_upload_size();
    }

    /*
    
    This ensures that the user only saves the slug in the database, not the full URL. Mostly used for legacy compatibility
    since we used to save the full URL in the database and now we don't.
    
    */
    public static function get_page_slug_without_home_url($saved_permalink, $default_permalink, $db_col) {
        
        $home_url = \home_url();
        $DB_Settings_General = \ShopWP\Factories\DB\Settings_General_Factory::build();

        $parts = explode($home_url, $saved_permalink);

        $parts_without_empty = array_values(array_filter($parts, function($v) {
            if ($v === '') {
                return false;
            }

            return !empty($v);
        }));

        if (!empty($parts_without_empty)) {
            $final_a = ltrim($parts_without_empty[0], '/');
            $final_a = rtrim($final_a, '/');

        } else {
            $final_a = $default_permalink;
        }

        // Should land here if user is setting a custom slug within the Permalinks page
        if ($final_a !== $default_permalink) {
            $final_slug = $final_a;
            
            $DB_Settings_General->update_col($db_col, $final_a);

        } else {
            $final_slug = $default_permalink;
        }   
        
        return ltrim($final_slug, '/');

    }

    public static function get_permalinks() {

        $DB_Settings_General = \ShopWP\Factories\DB\Settings_General_Factory::build();

        // Products
		$saved_products_permalink = $DB_Settings_General->col('url_products', 'string');
        $saved_products_permalink = ltrim($saved_products_permalink);
        $products_page_id = $DB_Settings_General->col('page_products', 'int');

        if (is_wp_error($products_page_id)) {
            $slug_of_default_products_page = SHOPWP_DEFAULT_PRODUCTS_PAGE_SLUG;
        } else {
            $slug_of_default_products_page = urldecode( ( $products_page_id > 0 && get_post( $products_page_id ) ) ? \get_page_uri( $products_page_id ) : SHOPWP_DEFAULT_PRODUCTS_PAGE_SLUG);
        }
        
        $products_final_slug = self::get_page_slug_without_home_url($saved_products_permalink, $slug_of_default_products_page, 'url_products');

        // Collections
		$saved_collections_permalink = $DB_Settings_General->col('url_collections', 'string');
        $saved_collections_permalink = ltrim($saved_collections_permalink);
        $collections_page_id = $DB_Settings_General->col('page_collections', 'int');

        if (is_wp_error($collections_page_id)) {
            $slug_of_default_collections_page = SHOPWP_DEFAULT_COLLECTIONS_PAGE_SLUG;
        } else {
            $slug_of_default_collections_page = urldecode( ( $collections_page_id > 0 && get_post( $collections_page_id ) ) ? \get_page_uri( $collections_page_id ) : SHOPWP_DEFAULT_COLLECTIONS_PAGE_SLUG);
        }

        $collections_final_slug = self::get_page_slug_without_home_url($saved_collections_permalink, $slug_of_default_collections_page, 'url_collections');

        return [
            'products'      => $products_final_slug,
            'collections'   => $collections_final_slug
        ];

    }

    public static function sanitize_permalink( $value ) {
        
        global $wpdb;

        $value = $wpdb->strip_invalid_text_for_column( $wpdb->options, 'option_value', $value );

        if ( is_wp_error( $value ) ) {
            $value = '';
        }

        $value = str_replace(' ', '', $value);

        $value = esc_url_raw( trim($value) );
        $value = str_replace( 'http://', '', $value);

        return ltrim(untrailingslashit($value), '/');

    }

    public static function clean( $var ) {
        if ( is_array( $var ) ) {
            return array_map( self::clean, $var );
        } else {
            return is_scalar( $var ) ? \sanitize_text_field( $var ) : $var;
        }
    }    

    public static function convert_shortcode_into_settings($shortcode_string) {
        $attrs = explode(' ', $shortcode_string);

        $okok = array_filter($attrs, function($attr) {
            return str_contains($attr, '="');
        });

        $final = [];

        foreach ($okok as $attr) {
            $stuff = explode('=', $attr);

            $ok = [];

            $one = str_replace('"', '', $stuff[1]);

            $two = str_replace(']', '', $one);
            
            $final[$stuff[0]] = $two;

        }

        return $final;
    }
}
