<?php

namespace ShopWP;

use ShopWP\Utils;
use ShopWP\Messages;

if (!defined('ABSPATH')) {
    exit();
}

class Webhooks
{
    public $plugin_settings;
    public $Template_Loader;
    public $API_Webhooks;
    public $DB_Settings_Syncing;
    public $API_Wrapper;
    public $DB_Settings_General;
    public $Syncing_Status;

    public function __construct($plugin_settings, $Template_Loader, $API_Webhooks, $DB_Settings_Syncing, $API_Wrapper, $DB_Settings_General, $Syncing_Status)
    {
        
        $this->plugin_settings          = $plugin_settings;
        $this->Template_Loader          = $Template_Loader;
        $this->API_Webhooks             = $API_Webhooks;
        $this->DB_Settings_Syncing      = $DB_Settings_Syncing;
        $this->API_Wrapper              = $API_Wrapper;
        $this->DB_Settings_General      = $DB_Settings_General;
        $this->Syncing_Status           = $Syncing_Status;

        add_action('shopwp_expire_sync', [$this, 'cancel_all_bulk_operations'], 10, 1);
        add_action('rest_api_init', [$this, 'register_routes']);

    }

    public function register_routes() {
        $this->API_Wrapper->api_route('/syncing/cron/scheduleAndSync', 'POST', [$this, 'handle_schedule_cron_and_sync']);
        $this->API_Wrapper->api_route('/syncing/cron/schedule', 'POST', [$this, 'handle_schedule_cron']);
        $this->API_Wrapper->api_route('/syncing/cron/remove', 'POST', [$this, 'handle_remove_cron']);
        $this->API_Wrapper->api_route('/webhook/bulk', 'POST', [$this, 'on_bulk_webhook_callback']);
        $this->API_Wrapper->api_route('/syncing/complete', 'POST', [$this, 'on_syncing_complete']);
    }

    private function calculate_hmac($data, $shared_secret)
    {
        return base64_encode(hash_hmac('sha256', $data, $shared_secret, true));
    }

    public function webhook_verified($data, $hmac_header)
    {

        $domain_base64 = base64_encode($this->plugin_settings['connection']['domain']);
        $transient_name = 'swp_sk_' . $domain_base64;

        $found_sk = \get_transient($transient_name);

        if (empty($found_sk)) {

            $key_resp = \wp_remote_post("https://auth.wpshop.io/wp-json/wpshopify-auth/api/key", [
                'body' => [
                    'shopify_domain'    => $this->plugin_settings['connection']['domain'],
                    'access_token'      => $this->plugin_settings['connection']['api_password'],
                    'wordpress_domain'  => $_SERVER['HTTP_HOST']
                ]
            ]);

            if (is_wp_error($key_resp)) {

                $this->DB_Settings_Syncing->log($key_resp->get_error_message(), 'error');
                $this->DB_Settings_Syncing->expire_sync();

                return false;
            }

            $data_new = json_decode(wp_remote_retrieve_body($key_resp));

            if (empty($data_new)) {
                return false;
            }

            if (isset($data_new->code)) {
                return false;
            }

            if (!isset($data_new->shared_secret)) {
                return false;
            }

            $shared_secret = $data_new->shared_secret;

            // Must have an active connection even if allow insecure is checked
            if (empty($shared_secret) || empty($hmac_header)) {
                $this->DB_Settings_Syncing->log(__('Empty SS, exiting ...', 'shopwp'), 'error');
                return false;
            }

            // Cache the sk for 1 day
            $result = \set_transient($transient_name, base64_encode($shared_secret), 24 * HOUR_IN_SECONDS);

        } else {

            $shared_secret = base64_decode($found_sk);
        }

        $calculated_hmac = $this->calculate_hmac($data, $shared_secret);

        if (empty($hmac_header)) {
            $this->DB_Settings_Syncing->log(__('Empty HMAC Header, exiting ...', 'shopwp'), 'error');
            return false;
        }

        $equals = hash_equals($hmac_header, $calculated_hmac);

        return $equals;
    }

    public function get_header_hmac()
    {
        $_SERVER = \wp_unslash($_SERVER);

        if (isset($_SERVER[SHOPWP_SHOPIFY_HEADER_VERIFY_WEBHOOKS])) {
            return $_SERVER[SHOPWP_SHOPIFY_HEADER_VERIFY_WEBHOOKS];
        }
    }

    public function get_callback_name_from_topic($topic)
    {
        return str_replace('/', '_', $topic) . '_callback';
    }

    public function before_webhook_process()
    {
        http_response_code(200);

        try {
            require_once ABSPATH . '/wp-admin/includes/class-wp-filesystem-base.php';
            require_once ABSPATH . '/wp-admin/includes/class-wp-filesystem-direct.php';

            $wp_filesystem = new \WP_Filesystem_Direct(null);

            $data = $wp_filesystem->get_contents('php://input');

            if (empty($data)) {

                $this->DB_Settings_Syncing->log(__('WP_Filesystem class is unavailable, using PHP file_get_contents() instead.', 'shopwp'), 'warning');
                $data = \file_get_contents("php://input");
            }

            return $data;
            
        } catch (\Throwable $th) {

            $this->DB_Settings_Syncing->log($th->getMessage(), 'error');
            $this->DB_Settings_Syncing->expire_sync();

            return false;
        }

    }

    public function handle_schedule_cron_and_sync($request) {

        $this->DB_Settings_Syncing->update_col('currently_active_tool', 'automaticSync');

        $this->DB_Settings_Syncing->toggle_syncing(1);
        $this->DB_Settings_Syncing->reset_syncing_notices();
        $this->DB_Settings_Syncing->reset_all_syncing_status();
        $this->DB_Settings_Syncing->reset_all_syncing_totals();
        $this->DB_Settings_Syncing->delete_log();

        // Gets current cron time, or creates new cron
        $next_cron_time = $this->API_Webhooks->maybe_schedule_cron('shopwp_auto_sync_cron_callback', 'shopwp_auto_sync_recheck_interval');

        return $this->Syncing_Status->get_syncing_status_data();

    }

    public function handle_schedule_cron($request) {

        // Gets current cron time, or creates new cron
        $next_cron_time = $this->API_Webhooks->maybe_schedule_cron('shopwp_auto_sync_cron_callback', 'shopwp_auto_sync_recheck_interval');

        return \wp_send_json_success($next_cron_time);

    }

    public function handle_remove_cron($request) {

        $unscheduled = $this->API_Webhooks->unschedule_all_crons();

        return wp_send_json_success($unscheduled);

    }

    public function on_syncing_complete($request) {

        $data = $request->get_param('data');

        $this->Template_Loader
                ->set_template_data($data, 'data', false)
                ->get_template_part('webhooks/syncing/complete');

    }


    public function is_valid_webhook($json_data)
    {

        $hmac = $this->get_header_hmac();

        if ($this->webhook_verified($json_data, $hmac)) {
            return true;
        }

        return false;
    }

    public function cancel_all_bulk_operations($cancel_auto_sync = false) {
    
        if ($cancel_auto_sync) {
            $unscheduled = $this->API_Webhooks->unschedule_all_crons();
        }

        $current_bulk_id = $this->DB_Settings_Syncing->col('bulk_op_id', 'string');

        if (!empty($current_bulk_id)) {

            $this->DB_Settings_Syncing->log(__('Canceling bulk operation: ', 'shopwp') . $current_bulk_id);

            $cancel_bulk_op_response = $this->API_Webhooks->admin_api_cancel_bulk_operation($current_bulk_id);

            if (is_wp_error($cancel_bulk_op_response)) {

                $this->DB_Settings_Syncing->log($cancel_bulk_op_response->get_error_message(), 'error');
                $this->DB_Settings_Syncing->update_col('bulk_op_id', '');

            } else {    
                $this->DB_Settings_Syncing->log(__('Finished canceling bulk operation ', 'shopwp') . $current_bulk_id . __(' with status: ', 'shopwp') . $cancel_bulk_op_response->status);
            }
        }

    }

    public function get_between_data($input_str, $start,$end) {
        $result_array = explode($start, $input_str);
        
        if (isset($result_array[1])) {
            $result_array = explode($end, $result_array[1]);
            return $result_array[0];
        }
        return '';
    }

    /* 
    
    Fires after receiving a webhook response from Shopify
    
    */
    public function on_bulk_webhook_callback() {

        $json_data                  = $this->before_webhook_process();
        $decoded                    = json_decode($json_data);
        $allow_insecure_default     = $this->DB_Settings_General->col('allow_insecure_webhooks', 'bool');
        $allow_insecure_webhooks    = apply_filters('shopwp_skip_bulk_webhook_ver', $allow_insecure_default);

        if (!$allow_insecure_webhooks && !$this->is_valid_webhook($json_data)) {

            $this->DB_Settings_Syncing->log(__('The webhook from Shopify is either invalid or expired.', 'shopwp'), 'error');
            $this->DB_Settings_Syncing->expire_sync();

            return false;
        }

        if (property_exists($decoded, 'admin_graphql_api_id')) {

            $msg = __('Received bulk data file from Shopify. Downloading ...', 'shopwp');

            $this->DB_Settings_Syncing->log($msg);
            $this->DB_Settings_Syncing->set_current_syncing_step_text($msg);

            \set_time_limit(0);

            $data = $this->API_Webhooks->admin_api_get_bulk_operation($decoded->admin_graphql_api_id);

            $current_webhook_id = $this->DB_Settings_Syncing->col('bulk_op_webhook_subscription_id', 'string');

            if (!empty($current_webhook_id)) {

                $remove_sub_response = $this->API_Webhooks->admin_api_delete_bulk_webhook_subscription($current_webhook_id);

                if (is_wp_error($remove_sub_response)) {
                    
                    $this->DB_Settings_Syncing->log($remove_sub_response->get_error_message(), 'error');
                    return;

                } else {
                    $this->DB_Settings_Syncing->update_col('bulk_op_webhook_subscription_id', '');
                }
            }

            $this->DB_Settings_Syncing->update_col('bulk_op_id', '');

            if (empty($data->url)) {

                if (isset($decoded->error_code)) {
                    $error_code = 'Error code: ' . $decoded->error_code;
                } else {
                    $error_code = '';
                }
                
                $this->DB_Settings_Syncing->log(__('Tried running the bulk query but no data was found. Nothing was synced. ', 'shopwp') . $error_code, 'error');

                $this->DB_Settings_Syncing->expire_sync();
                return;
            }

            $response_obj = \wp_remote_get($data->url, [
                'timeout' => 120
            ]);

            if (\is_wp_error($response_obj)) {
                $this->DB_Settings_Syncing->log($response_obj->get_error_message(), 'error');
                return;
            }

            $json = \wp_remote_retrieve_body($response_obj);

            $this->DB_Settings_Syncing->log(__('Finished downloading data file', 'shopwp'));

            $stuff = array_filter(explode(PHP_EOL, $json));

            rsort($stuff);

            $this->DB_Settings_Syncing->log(__('Processing downloaded data file', 'shopwp'));

            $this->DB_Settings_Syncing->update_col('syncing_totals_products', count($stuff));


            $this->DB_Settings_Syncing->log(__('Ready to process ', 'shopwp') . count($stuff) . __(' items, let\'s go!', 'shopwp'));

            $this->Template_Loader
                ->set_template_data($stuff, 'data', false)
                ->get_template_part('webhooks/bulk/products');

        } else {
            $this->DB_Settings_Syncing->log(__('The webhook response from Shopify did not contain a admin_graphql_api_id key. Expiring sync ...', 'shopwp'), 'error');
            $this->DB_Settings_Syncing->expire_sync();

            return false;
        }

    }

}
