<?php

namespace ShopWP\DB;

use ShopWP\Utils;
use ShopWP\CPT;

if (!defined('ABSPATH')) {
    exit();
}

class Collections extends \ShopWP\DB
{
    public $table_name_suffix;
    public $table_name;
    public $version;
    public $primary_key;
    public $lookup_key;
    public $cache_group;
    public $type;

    public function __construct()
    {
        $this->table_name_suffix = SHOPWP_TABLE_NAME_COLLECTIONS;
        $this->table_name = $this->get_table_name();
        $this->version = '1.0';
        $this->primary_key = 'id';
        $this->lookup_key = SHOPWP_COLLECTIONS_LOOKUP_KEY;
        $this->cache_group = 'shopwp_db_collections';
        $this->type = 'collection';
    }

    public function get_columns()
    {
        return [
            'id'                            => '%d',
            'postId'                        => '%d',
            'collectionId'                  => '%s',
            'description'                   => '%s',
            'descriptionHtml'               => '%s',
            'feedback'                      => '%s',
            'handle'                        => '%s',
            'image'                         => '%s',
            'metafield'                     => '%s',
            'productsCount'                 => '%d',
            'publishedOnCurrentPublication' => '%d',
            'ruleSet'                       => '%s',
            'seo'                           => '%s',
            'sortOrder'                     => '%s',
            'title'                         => '%s',
            'translations'                  => '%s',
            'updatedAt'                     => '%s'
        ];
    }

    /*

    Columns that should remain integers during casting.
    We need to check against this when retrieving data since MYSQL 
    converts all cols to strings upon retrieval. 

    */
    public function cols_that_should_remain_ints()
    {
        return ['id', 'postId', 'productsCount'];
    }

    /*

	Default Smart Collections Query

	*/
    public function get_default_query()
    {
        global $wpdb;

        return array(
            'where' => '',
            'groupby' => '',
            'join' =>
                ' INNER JOIN ' .
                $this->table_name .
                ' smart ON ' .
                $wpdb->posts .
                '.ID = smart.post_id',
            'orderby' => '',
            'distinct' => '',
            'fields' => 'smart .*',
            'limits' => ''
        );
    }

    public function mod_before_change($collection, $post_id = false)
    {
        $collection_copy = $this->copy($collection);

        $collection_copy = $this->maybe_rename_to_lookup_key($collection_copy);
        $collection_copy = Utils::flatten_image_prop($collection_copy);

        if ($post_id) {
            $collection_copy = CPT::set_post_id($collection_copy, $post_id);
        }

        // Important. If handle doesn't match post_name, the product won't show
        $collection_copy->post_name = sanitize_title($collection_copy->handle);

        if ($collection_copy->collectionId) {
            $collection_copy->collectionId = \str_replace("gid://shopify/Collection/", "", $collection_copy->collectionId);
        }

        unset($collection_copy->__parentId);

        return $collection_copy;
    }

    public function insert_collection($collection)
    {
        return $this->insert($collection);
    }

    public function update_collection($collection)
    {
        return $this->update(
            $this->lookup_key,
            $this->get_lookup_value($collection),
            $collection
        );
    }

    public function delete_collection($collection)
    {
        return $this->delete_rows(
            $this->lookup_key,
            $this->get_lookup_value($collection)
        );
    }

    public function delete_collections_from_collection_id($collection_id)
    {
        return $this->delete_rows($this->lookup_key, $collection_id);
    }

    public function has_collection($maybe_collection)
    {
        if (
            is_object($maybe_collection[0]) &&
            property_exists($maybe_collection[0], 'collection_id')
        ) {
            return true;
        }

        return false;
    }

    public function is_smart_collection($collection)
    {
        return Utils::has($collection, 'rules') ? true : false;
    }

    public function get_post_id_from_collection($collection)
    {
        $args = array(
            'post_type' => 'wps_products',
            'meta_query' => array(
                array(
                    'key' => 'collection_id',
                    'value' => $collection->id,
                    'compare' => '=',
                )
            )
        );
        
        $collection_found = \get_posts($args);

        if (empty($collection_found) || \is_wp_error($collection_found)) {
            return false;
        }

        return $collection_found[0]->ID;
    }

    public function normalize_collection_columns($collection) {

        if (property_exists($collection, 'collection_id')) {
            $collection->collectionId = $collection->collection_id;
            unset($collection->collection_id);
        }

        if (property_exists($collection, 'updated_at')) {
            $collection->updatedAt = $collection->updated_at;
            unset($collection->updated_at);
        }

        if (property_exists($collection, 'body_html')) {
            $collection->description = $collection->body_html;
            $collection->descriptionHtml = $collection->body_html;
            unset($collection->body_html);
        }

        if (property_exists($collection, 'sort_order')) {
            $collection->sortOrder = $collection->sort_order;
            unset($collection->sort_order);
        }

        if (!empty($collection->image)) {
            $collection->image->url = $collection->image->src;
            $collection->image->createdAt = $collection->image->created_at;
            
            unset($collection->image->src);
            unset($collection->image->created_at);
        }

        return $collection;
    }

    /*

	Creates a table query string

	*/
    public function create_table_query($table_name = false)
    {
        if (!$table_name) {
            $table_name = $this->table_name;
        }

        $collate = $this->collate();

        return "CREATE TABLE $table_name (
            id bigint(100) unsigned NOT NULL AUTO_INCREMENT,
            postId bigint(100) DEFAULT 0, 
            collectionId varchar(255) DEFAULT '',
            description longtext NOT NULL DEFAULT '',
            descriptionHtml longtext NOT NULL DEFAULT '',
            feedback longtext NOT NULL DEFAULT '',
            handle varchar(255) DEFAULT '',
            image MEDIUMTEXT DEFAULT '',
            metafield MEDIUMTEXT DEFAULT '',
            productsCount bigint(100) unsigned DEFAULT 0,
            publishedOnCurrentPublication tinyint(1) DEFAULT 0,
            ruleSet longtext DEFAULT '',
            seo longtext NOT NULL DEFAULT '',
            sortOrder varchar(255) DEFAULT '',
            title mediumtext DEFAULT '',
            translations mediumtext DEFAULT '',
            updatedAt datetime DEFAULT '0000-00-00 00:00:00',
			PRIMARY KEY  (id)
		) ENGINE=InnoDB $collate";
    }
}