<?php

namespace ShopWP\DB;

use ShopWP\Utils;

if (!defined('ABSPATH')) {
    exit();
}

class Customers extends \ShopWP\DB
{
    public $table_name_suffix;
    public $table_name;
    public $version;
    public $primary_key;
    public $lookup_key;
    public $cache_group;
    public $type;

    public function __construct()
    {
        // Table info
        $this->table_name_suffix = SHOPWP_TABLE_NAME_CUSTOMERS;
        $this->table_name = $this->get_table_name();
        $this->version = '1.0';
        $this->primary_key = 'id';
        $this->lookup_key = 'email';
        $this->cache_group = 'shopwp_db_customers';
        $this->type = 'customer';

    }

    public function get_columns()
    {
        return [
            'id' => '%d',
            'customerId' => '%s',
            'addresses' => '%s',
            'amountSpent' => '%s',
            'averageOrderAmountV2' => '%s',
            'canDelete' => '%d',
            'createdAt' => '%s',
            'defaultAddress' => '%s',
            'displayName' => '%s',
            'email' => '%s',
            'emailMarketingConsent' => '%s',
            'firstName' => '%s',
            'hasTimelineComment' => '%d',
            'image' => '%s',
            'lastName' => '%s',
            'lastOrder' => '%s',
            'legacyResourceId' => '%s',
            'lifetimeDuration' => '%s',
            'locale' => '%s',
            'market' => '%s',
            'metafield' => '%s',
            'multipassIdentifier' => '%s',
            'note' => '%s',
            'numberOfOrders' => '%d',
            'phone' => '%s',
            'productSubscriberStatus' => '%s',
            'smsMarketingConsent' => '%s',
            'state' => '%s',
            'tags' => '%s',
            'taxExempt' => '%s',
            'taxExemptions' => '%s',
            'unsubscribeUrl' => '%s',
            'updatedAt' => '%s',
            'userId' => '%d',
            'validEmailAddress' => '%d',
            'verifiedEmail' => '%d',
        ];
    }

    /*

   Columns that should remain integers during casting.
   We need to check against this when retrieving data since MYSQL 
   converts all cols to strings upon retrieval. 

   */
    public function cols_that_should_remain_ints()
    {
        return [
            'id',
            'numberOfOrders',
            'userId'
        ];
    }

    public function mod_before_change($customer)
    {

        $customer_copy = $this->copy($customer);

        if (isset($customer_copy->id)) {
            $customer_copy->customerId = \str_replace("gid://shopify/Customer/", "", $customer_copy->id);
        }

        unset($customer_copy->id);

        return $customer_copy;
        
    }

    /*

	The modify options used for inserting / updating / deleting

	*/
    public function modify_options(
        $shopify_item,
        $item_lookup_key = SHOPWP_PRODUCTS_LOOKUP_KEY
    ) {
        return [
            'item' => $shopify_item,
            'item_lookup_key' => $item_lookup_key,
            'item_lookup_value' => $shopify_item->id,
            'prop_to_access' => 'customers',
            'change_type' => 'customer'
        ];
    }

    /*

	Inserts single customer

	*/
    public function insert_customer($customer)
    {
        return $this->insert($customer);
    }

    /*

	Updates a single variant

	*/
    public function update_customer($customer)
    {
        return $this->update($this->lookup_key, $customer->id, $customer);
    }

    /*

	Deletes a single customer

	*/
    public function delete_customer($customer)
    {
        return $this->delete_rows(
            $this->lookup_key,
            $this->get_lookup_value($customer)
        );
    }

    public function delete_customer_by_user_id($user_id)
    {
        return $this->delete_rows('userId', $user_id);
    }

    /*

	Gets all customers

	*/
    public function get_customers()
    {
        return $this->get_all_rows();
    }

    public function get_customer_access_token_from_user_id($user_id)
    {
        $token = $this->select_in_col('access_token', 'userId', $user_id);

        if (empty($token)) {
            return false;
        }

        return $token[0];
    }

    public function convert_graph_id_to_rest_id($graph_id, $type)
    {
        return explode($type, base64_decode($graph_id))[1];
    }

    public function get_rest_id_from_graph_id($customer_graph_id)
    {
        return $this->convert_graph_id_to_rest_id(
            $customer_graph_id,
            'Customer/'
        );
    }

    public function get_user_id_from_customer_id($customer_graph_id)
    {
        $customer_rest_id = $this->get_rest_id_from_graph_id(
            $customer_graph_id
        );

        $user_id = $this->select_in_col(
            'userId',
            'customerId',
            $customer_rest_id
        );

        if (empty($user_id)) {
            return false;
        }

        return $user_id[0];
    }

    public function is_user_customer($user)
    {
        if (!get_object_vars($user->data) || !property_exists($user, 'roles')) {
            return false;
        }

        if (in_array('wpshopify_customer', $user->roles)) {
            return true;
        }

        return false;
    }

    /*

	Creates a table query string

	*/
    public function create_table_query($table_name = false)
    {
        if (!$table_name) {
            $table_name = $this->table_name;
        }

        $collate = $this->collate();

        return "CREATE TABLE $table_name (
            id bigint(100) unsigned NOT NULL AUTO_INCREMENT,
            customerId varchar(255) DEFAULT '',
            addresses MEDIUMTEXT DEFAULT '',
            amountSpent varchar(255) DEFAULT '',
            averageOrderAmountV2 varchar(255) DEFAULT '',
            canDelete tinyint(1) DEFAULT 0,
            createdAt datetime DEFAULT '0000-00-00 00:00:00',
            defaultAddress MEDIUMTEXT DEFAULT '',
            displayName mediumtext DEFAULT '',
            email varchar(255) DEFAULT '',
            emailMarketingConsent varchar(255) DEFAULT '',
            firstName MEDIUMTEXT DEFAULT '',
            hasTimelineComment tinyint(1) DEFAULT 0,
            image MEDIUMTEXT DEFAULT '',
            lastName mediumtext DEFAULT '',
            lastOrder MEDIUMTEXT DEFAULT '',
            legacyResourceId varchar(255) DEFAULT '',
            lifetimeDuration varchar(255) DEFAULT '',
            locale varchar(255) DEFAULT '',
            market MEDIUMTEXT DEFAULT '',
            metafield MEDIUMTEXT DEFAULT '',
            multipassIdentifier varchar(255) DEFAULT '',
            note MEDIUMTEXT DEFAULT '',
            numberOfOrders bigint(100) DEFAULT 0,
            phone varchar(255) DEFAULT '',
            productSubscriberStatus varchar(255) DEFAULT '',
            smsMarketingConsent MEDIUMTEXT DEFAULT '',
            state MEDIUMTEXT DEFAULT '',
            tags MEDIUMTEXT DEFAULT '',
            taxExempt varchar(255) DEFAULT '',
            taxExemptions MEDIUMTEXT DEFAULT '',
            unsubscribeUrl MEDIUMTEXT DEFAULT '',
            updatedAt datetime DEFAULT '0000-00-00 00:00:00',
            userId bigint(100) DEFAULT 0,
            validEmailAddress tinyint(1) DEFAULT 0,
            verifiedEmail tinyint(1) DEFAULT 0,
			PRIMARY KEY  (id)
		) ENGINE=InnoDB $collate";
    }
}
