<?php

namespace ShopWP\DB;

use ShopWP\Utils;
use ShopWP\Utils\URLs;

if (!defined('ABSPATH')) {
    exit();
}

class Images extends \ShopWP\DB
{

    public $table_name_suffix;
    public $table_name;
    public $version;
    public $primary_key;
    public $lookup_key;
    public $cache_group;
    public $type;

    public function __construct()
    {
        $this->table_name_suffix = SHOPWP_TABLE_NAME_IMAGES;
        $this->table_name = $this->get_table_name();
        $this->version = '1.0';
        $this->primary_key = 'id';
        $this->lookup_key = 'imageId';
        $this->cache_group = 'shopwp_db_images';
        $this->type = 'image';
    }

    public function get_columns()
    {
        return [
            'id'            => '%d',
            'imageId'       => '%s',
            'collectionId'  => '%s',
            'productId'     => '%s',
            'altText'       => '%s', 
            'width'         => '%s',
            'height'        => '%s',
            'metafield'     => '%s',
            'url'           => '%s',
            'isFeatured'    => '%d'
        ];
    }

    public function cols_that_should_remain_ints()
    {
        return [
            'id'
        ];
    }

    public function modify_options(
        $shopify_item,
        $item_lookup_key = SHOPWP_PRODUCTS_LOOKUP_KEY
    ) {
        return [
            'item' => $shopify_item,
            'item_lookup_key' => $item_lookup_key,
            'item_lookup_value' => $shopify_item->id,
            'prop_to_access' => 'images',
            'change_type' => 'image'
        ];
    }

    public function mod_before_change($image)
    {
        $image_copy = $this->copy($image);
        $image_copy = $this->maybe_rename_to_lookup_key($image_copy);
        
        if (isset($image_copy->productId)) {
            $image_copy->productId = \str_replace("gid://shopify/Product/", "", $image_copy->productId);
        }

        if (isset($image_copy->collectionId)) {
            $image_copy->collectionId = \str_replace("gid://shopify/Collection/", "", $image_copy->collectionId);
        }

        if (isset($image_copy->imageId)) {

            if (Utils::str_contains($image_copy->imageId, 'ProductImage')) { 
                $image_copy->imageId = \str_replace("gid://shopify/ProductImage/", "", $image_copy->imageId);

            } else if (Utils::str_contains($image_copy->imageId, 'CollectionImage')) {
                $image_copy->imageId = \str_replace("gid://shopify/CollectionImage/", "", $image_copy->imageId);
            }
        }

        unset($image_copy->__parentId);

        return $image_copy;
    }

    public function update_image($image)
    {
        return $this->update(
            $this->lookup_key,
            $this->get_lookup_value($image),
            $image
        );
    }

    public function delete_image($image)
    {
        return $this->delete_rows(
            $this->lookup_key,
            $this->get_lookup_value($image)
        );
    }

    public function delete_images_from_product_id($product_id, $post_id = false)
    {
        if (!empty($post_id)) {

            $attachment_id = \get_post_thumbnail_id($post_id);
            $attachment_del = \wp_delete_attachment($attachment_id);
        }

        return $this->delete_rows(SHOPWP_PRODUCTS_LOOKUP_KEY, $product_id);
    }

    public function get_images_from_product_id($product_id)
    {
        return $this->get_rows(SHOPWP_PRODUCTS_LOOKUP_KEY, $product_id, ['url', 'altText', 'width', 'height', 'isFeatured']);
    }

    public function query_all_images($table_name)
    {
        return 'SELECT images.src, images.post_id, images.alt FROM ' . $table_name . ' as images WHERE images.position = 1';
    }

    public function get_all_images()
    {
        global $wpdb;

        $query = $this->query_all_images($this->table_name);

        return $wpdb->get_results($query);
    }

    public function remove_image($image_id) {
        return \wp_delete_attachment($image_id, true);
    }

    public function delete_all_images($images)
    {
        if (empty($images)) {
            return [];
        }

        foreach ($images as $img_id) {
            $results[] = $this->remove_image($img_id->ID);
        }

        return $results;
    }

    public function get_all_plugin_attachments()
    {
        $args = [
            'posts_per_page' => -1,
            'post_type' => 'attachment',
            'post_status' => 'inherit',
            'no_found_rows' => true,
            'meta_query' => [
                [
                    'key' => '_wp_attachment_metadata',
                    'value' => 'shopwp-media',
                    'compare' => 'LIKE'
                ]
            ]
        ];

        $images = new \WP_Query($args);

        if (empty($images) || \is_wp_error($images)) {
            return [];
        }

        $img_ids = $images->posts;

        if ($img_ids && !empty($img_ids)) {
            return $img_ids;

        } else {
            return [];
        }

    }

    public function query_get_all_plugin_attachments_meta($posts_table, $post_meta_table)
    {
        return "SELECT " . $post_meta_table . ".* FROM " . $posts_table . " INNER JOIN " . $post_meta_table . " ON (" . $posts_table . ".ID = " . $post_meta_table . ".post_id ) WHERE 1=1 AND (( " . $post_meta_table . ".meta_key = '_wp_attachment_metadata' AND " . $post_meta_table . ".meta_value LIKE '%shopwp-media%' )) AND " . $posts_table . ".post_type = 'attachment' AND ((" . $posts_table . ".post_status = 'inherit')) GROUP BY " . $posts_table . ".ID";

    }

    public function get_all_plugin_attachments_meta()
    {
        global $wpdb; 

        $posts_table = $wpdb->prefix . 'posts';
        $post_meta_table = $wpdb->prefix . 'postmeta';

        $query = $this->query_get_all_plugin_attachments_meta($posts_table, $post_meta_table);

        return $wpdb->get_results($query);
    }

    /*

	Creates a table query string

	*/
    public function create_table_query($table_name = false)
    {
        if (!$table_name) {
            $table_name = $this->table_name;
        }

        $collate = $this->collate();

        return "CREATE TABLE $table_name (
            id bigint(100) unsigned NOT NULL AUTO_INCREMENT,
            imageId varchar(255) DEFAULT 0,
            collectionId varchar(255) DEFAULT 0,
            productId varchar(255) DEFAULT 0,
            altText mediumtext DEFAULT '', 
            width varchar(255) DEFAULT 0,
            height varchar(255) DEFAULT 0,
            metafield mediumtext DEFAULT '',
            url longtext DEFAULT '',
            isFeatured  tinyint(1) DEFAULT 0,
			PRIMARY KEY  (id)
		) ENGINE=InnoDB $collate";
    }
}
