<?php

namespace ShopWP\DB;

use ShopWP\Utils;
use ShopWP\Utils\Data as Utils_Data;
use ShopWP\Transients;
use ShopWP\CPT;

if (!defined('ABSPATH')) {
    exit();
}

class Posts extends \ShopWP\DB
{
    public function __construct() {}

    public function delete_posts_by_type($post_type)
    {
        global $wpdb;

        $wp_posts = $wpdb->prefix . SHOPWP_TABLE_NAME_WP_POSTS;
        $wp_term = $wpdb->prefix . SHOPWP_TABLE_NAME_WP_TERM_RELATIONSHIPS;
        $meta = $wpdb->prefix . SHOPWP_TABLE_NAME_WP_POSTMETA;

        $query = $wpdb->prepare('DELETE posts, pt, pm FROM ' . $wp_posts . ' posts LEFT JOIN ' . $wp_term . ' pt ON pt.object_id = posts.ID LEFT JOIN ' . $meta . ' pm ON pm.post_id = posts.ID WHERE posts.post_type = %s', $post_type);

        return $this->query($query);
    }

    public function delete_tax_raw($taxonomy_name) {

        global $wpdb;
        $results = [];

        # Deletes all the terms associated to the taxonomy.
        #
        # The inner query collects from the 'term_taxonomy' 
        # table all the ids of the terms to delete.
        $results[] = $wpdb->query( 
        "DELETE FROM {$wpdb->terms} WHERE {$wpdb->terms}.term_id IN (
            SELECT tt.term_id FROM {$wpdb->term_taxonomy} AS tt
            WHERE tt.taxonomy = '{$taxonomy_name}'
        )");

        # Deletes the information used by WordPress 
        # to link each post to their terms.
        #
        # Here too, a nested query is needed, 
        # and the 'term_taxonomy' table is yet again 
        # the source of the data passed to the outer query.
        # But in this case to be collected by 
        # the inner query are the ids of 
        # the Relationships to delete.
        $results[] = $wpdb->query( 
        "DELETE FROM {$wpdb->term_relationships} WHERE {$wpdb->term_relationships}.term_taxonomy_id IN (
            SELECT tt.term_taxonomy_id FROM {$wpdb->term_taxonomy} AS tt
            WHERE tt.taxonomy = '{$taxonomy_name}'
        )");

        # Deletes the base information about the taxonomy. 
        $results[] = $wpdb->query("DELETE FROM {$wpdb->term_taxonomy} WHERE taxonomy = '{$taxonomy_name}'");

        return $results;

    }

    /*

	Delete all posts

	*/
    public function delete_all_posts()
    {

        $results = [];

        $num_of_posts = \wp_count_posts(SHOPWP_PRODUCTS_POST_TYPE_SLUG);

        if (empty($num_of_posts->publish) || $num_of_posts->publish < 500) {
            $results['shopify_tags']            = $this->delete_taxonomies('shopify_tags');
            $results['shopify_collections']     = $this->delete_taxonomies('shopify_collections');
            $results['shopify_vendor']          = $this->delete_taxonomies('shopify_vendor');
            $results['shopify_product_type']    = $this->delete_taxonomies('shopify_product_type');
            
        } else {
            $results['shopify_tags']            = $this->delete_tax_raw('shopify_tags');
            $results['shopify_collections']     = $this->delete_tax_raw('shopify_collections');
            $results['shopify_vendor']          = $this->delete_tax_raw('shopify_vendor');
            $results['shopify_product_type']    = $this->delete_tax_raw('shopify_product_type');
        }

        $results[
            SHOPWP_PRODUCTS_POST_TYPE_SLUG
        ] = $this->delete_posts_by_type(SHOPWP_PRODUCTS_POST_TYPE_SLUG);

        $results[
            SHOPWP_COLLECTIONS_POST_TYPE_SLUG
        ] = $this->delete_posts_by_type(SHOPWP_COLLECTIONS_POST_TYPE_SLUG);

        return $results;

    }

    /*

	Delete posts by ids

	*/
    public function delete_posts_by_ids($post_ids)
    {
        global $wpdb;

        if (!is_array($post_ids)) {
            $post_ids = Utils::maybe_wrap_in_array($post_ids);
        }

        if (empty($post_ids)) {
            return false;
        }

        // how many entries will we select?
        $how_many = count($post_ids);

        // prepare the right amount of placeholders
        $placeholders = array_fill(0, $how_many, '%d');

        // $format = '%d, %d, %d, %d, %d, [...]'
        $format = Utils::convert_to_comma_string($placeholders);

        $wp_posts = $wpdb->prefix . SHOPWP_TABLE_NAME_WP_POSTS;
        $wp_term = $wpdb->prefix . SHOPWP_TABLE_NAME_WP_TERM_RELATIONSHIPS;
        $meta = $wpdb->prefix . SHOPWP_TABLE_NAME_WP_POSTMETA;

        $query = "DELETE posts, pt, pm FROM " . $wp_posts . " posts LEFT JOIN " . $wp_term . " pt ON pt.object_id = posts.ID LEFT JOIN " . $meta . " pm ON pm.post_id = posts.ID WHERE posts.ID IN(" . $format . ")";

        $query_prepared = $wpdb->prepare($query, $post_ids);

        return $wpdb->query($query_prepared);
    }

    /*

	Delete taxonomies

	*/
    public function delete_taxonomies($type)
    {

        if (!\taxonomy_exists($type)) {
            return;
        }
        
        $deletions = [];

        $terms = \get_terms([
            'taxonomy' => $type,
            'hide_empty' => false,
            'fields' => 'ids'
        ]);

        foreach ($terms as $term_id) {
            $deletions[] = \wp_delete_term($term_id, $type);
        }

        return $deletions;
    }

    /*

	Delete the synced Shopify data
	$type = 'all', $items = false

	*/
    public function delete_posts($params = [])
    {
        $results = [];

        if (empty($params)) {
            $delete_all = $this->delete_all_posts();

            return $delete_all;
        }

        if (!empty($params['ids'])) {
            $results['delete_posts_ids'] = $this->delete_posts_by_ids(
                $params['ids']
            );
            return $results;
        }

        if (!empty($params['post_type'])) {
            $results['delete_posts_type'] = $this->delete_posts_by_type(
                $params['post_type']
            );
            return $results;
        }
    }

    public function get_from_post_name_query_prepared($table_name, $post_name)
    {
        return 'SELECT items.* FROM ' . $table_name . ' as items WHERE items.post_name = ' . $post_name;
    }

    /*

	Finds a given item based on its post ID

	*/
    public function get_from_post_name($table_name = false, $post_name = false)
    {
        global $wpdb;

        if (!$table_name) {
            return;
        }

        if ($post_name === false) {
            return;
        }

        $query = $this->get_from_post_name_query_prepared($post_name, $table_name);

        return $wpdb->get_row($query);
    }
}
