<?php

namespace ShopWP\DB;

use ShopWP\CPT;
use ShopWP\Utils;

if (!defined('ABSPATH')) {
    exit();
}

class Products extends \ShopWP\DB
{
    public $table_name_suffix;
    public $table_name;
    public $version;
    public $primary_key;
    public $lookup_key;
    public $cache_group;
    public $type;

    public function __construct()
    {
        $this->table_name_suffix = SHOPWP_TABLE_NAME_PRODUCTS;
        $this->table_name = $this->get_table_name();
        $this->version = '1.0';
        $this->primary_key = 'id';
        $this->lookup_key = 'productId';
        $this->cache_group = 'shopwp_db_products';
        $this->type = 'product';
    }

    public function get_columns()
    {
        return [
            'id' => '%d',
            'postId' => '%d',
            'productId' => '%s',
            'availablePublicationCount' => '%d',
            'contextualPricing' => '%s',
            'createdAt' => '%s',
            'customProductType' => '%s',
            'defaultCursor' => '%s',
            'description' => '%s',
            'descriptionHtml' => '%s',
            'featuredImage' => '%s',
            'featuredMedia' => '%s',
            'handle' => '%s',
            'hasOnlyDefaultVariant' => '%d',
            'hasOutOfStockVariants' => '%d',
            'isGiftCard' => '%d',
            'legacyResourceId' => '%s',
            'mediaCount' => '%d',
            'metafield' => '%s',
            'onlineStorePreviewUrl' => '%s',
            'onlineStoreUrl' => '%s',
            'options' => '%s',
            'priceRangeV2' => '%s',
            'privateMetafield' => '%s',
            'productType' => '%s',
            'publicationCount' => '%d',
            'publishedAt' => '%s',
            'publishedOnCurrentPublication' => '%s',
            'publishedOnPublication' => '%s',
            'requiresSellingPlan' => '%d',
            'resourcePublicationOnCurrentPublication' => '%s',
            'sellingPlanGroupCount' => '%d',
            'seo' => '%s',
            'standardizedProductType' => '%s',
            'status' => '%s',
            'tags' => '%s',
            'title' => '%s',
            'totalInventory' => '%d',
            'totalVariants' => '%d',
            'tracksInventory' => '%d',
            'translations' => '%s',
            'updatedAt' => '%s',
            'vendor' => '%s',
            'collections' => '%s',
        ];
    }

    /*

    Columns that should remain integers during casting.
    We need to check against this when retrieving data since MYSQL 
    converts all cols to strings upon retrieval. 

    */
    public function cols_that_should_remain_ints()
    {
        return [
            'id', 
            'postId',
            'availablePublicationCount',
            'mediaCount',
            'publicationCount',
            'sellingPlanGroupCount',
            'totalInventory',
            'totalVariants'
        ];

    }

    public function modify_options(
        $shopify_item,
        $item_lookup_key = 'productId'
    ) {
        return [
            'item' => $shopify_item,
            'item_lookup_key' => $item_lookup_key,
            'item_lookup_value' => $shopify_item->id,
            'prop_to_access' => 'products',
            'change_type' => 'product',
        ];
    }

    public function mod_before_change($product, $post_id = false)
    {

        $product_copy = $this->copy($product);
        
        $product = $this->maybe_rename_to_lookup_key($product_copy);
        // $product = Utils::flatten_image_prop($product);

        if (isset($product->productId)) {
            $product->productId = \str_replace("gid://shopify/Product/", "", $product->productId);
        }

        if ($post_id) {
            $product = CPT::set_post_id($product, $post_id);
        }

        if (empty($product->publishedAt)) {
            $product->publishedAt = '0000-00-00 00:00:00';
        }

        if (empty($product->createdAt)) {
            $product->createdAt = '0000-00-00 00:00:00';
        }

        if (empty($product->updatedAt)) {
            $product->updatedAt = '0000-00-00 00:00:00';
        }
        
        // Important. If handle doesn't match post_name, the product won't show
        // $product->post_name = sanitize_title($product->handle);

        unset($product->__parentId);

        return $product;
    }

    public function insert_product($product)
    {
        return $this->insert($product);
    }

    public function update_product($product)
    {
        return $this->update(
            $this->lookup_key,
            $this->get_lookup_value($product),
            $product
        );
    }

    public function delete_product($product)
    {
        return $this->delete_rows(
            $this->lookup_key,
            $this->get_lookup_value($product)
        );
    }

    public function delete_products_from_product_id($product_id)
    {
        return $this->delete_rows($this->lookup_key, $product_id);
    }

    public function get_product_from_product_id($product_id)
    {
        return $this->get_rows($this->lookup_key, $product_id);
    }

    public function return_vendor($product)
    {
        return $product->vendor;
    }

    public function return_product_type($product)
    {
        return $product->product_type;
    }

    public function get_unique_vendors()
    {
        $products = $this->get_all_rows();

        return array_values(
            array_unique(array_map([__CLASS__, 'return_vendor'], $products))
        );
    }

    public function get_unique_types()
    {
        $products = $this->get_all_rows();

        return array_values(
            array_unique(
                array_map([__CLASS__, 'return_product_type'], $products)
            )
        );
    }

    public function query_get_product_from_post_id($table_name, $post_id) {
        return 'SELECT products.* FROM ' . $table_name . ' as products WHERE products.postId = ' . $post_id;
    }

    public function get_product($post_id = null)
    {
        global $wpdb;

        if ($post_id === null) {
            $post_id = get_the_ID();
        }

        if (empty($post_id)) {
            return false;
        }

        $query_prepared = $this->query_get_product_from_post_id($this->table_name, $post_id);

        return $wpdb->get_row($query_prepared);

    }

    public function get_product_ids_query($limit = false)
    {
        global $wpdb;

        $table_name = $wpdb->prefix . SHOPWP_TABLE_NAME_PRODUCTS;

        if ($limit) {
            return 'SELECT product.productId FROM ' . $table_name . ' product LIMIT ' . $limit;

        } else {
            return 'SELECT product.productId FROM ' . $table_name . ' product';
        }

        return $main_query;
    }

    public function get_product_ids($limit = false)
    {
        global $wpdb;

        $query = $this->get_product_ids_query($limit);

        $results = $wpdb->get_results($query);

        return array_column($results, 'product_id');
    }

    /*

    Add Post ID To Product

     */
    public function add_post_id_to_product($product, $cpt_id)
    {
        $product->post_id = $cpt_id;

        return $product;
    }

    /*

    Assigns a post id to the product data

     */
    public function assign_post_id_to_product($post_id, $product_id)
    {
        global $wpdb;

        return $wpdb->update(
            $this->table_name,
            ['post_id' => $post_id],
            [$this->lookup_key => $product_id],
            ['%d'],
            ['%d']
        );
    }

    public function product_exists($product_id)
    {
        if (empty($product_id)) {
            return false;
        }

        $product_found = $this->get_row_by('productId', $product_id);

        if (empty($product_found)) {
            return false;
        } else {
            return true;
        }
    }

    /*

    Find a post ID from a product ID

     */
    public function get_post_id_from_product_id($product_id)
    {
        $product = $this->get_product_from_product_id($product_id);

        if (empty($product)) {
            return false;
        }

        return $product[0]->postId;
    }

    public function update_products($products)
    {
        $result = [];

        foreach ($products as $key => $product) {
            $result[] = $this->update(
                $this->lookup_key,
                $product['id'],
                $product
            );
        }

        return $result;
    }

    public function get_product_ids_from_titles($titles)
    {
        return $this->select_in_col('productId', 'title', $titles);
    }

    public function get_product_ids_from_vendors($vendors)
    {
        return $this->select_in_col('productId', 'vendor', $vendors);
    }

    public function get_product_ids_from_handles($handles)
    {
        return $this->select_in_col('productId', 'handle', $handles);
    }

    public function get_product_ids_from_types($types)
    {
        return $this->select_in_col('productId', 'product_type', $types);
    }

    public function get_product_ids_from_post_ids($post_ids)
    {   
        $result = [];

        foreach ($post_ids as $post_id) {
            $result[] = get_post_meta($post_id, 'productId', true);
        }

        $result = array_filter($result);

        if (empty($result)) {
            return false;
        }

        return $result;

    }

    public function get_product_ids_from_description($description)
    {
        return $this->select_like_col(
            'productId',
            'body_html',
            $description,
            $this->table_name
        );
    }

    public function get_products()
    {
        return $this->get_all_rows();
    }

    public function create_table_query($table_name = false)
    {
        if (!$table_name) {
            $table_name = $this->table_name;
        }

        $collate = $this->collate();

        return "CREATE TABLE $table_name (
            id bigint(100) unsigned NOT NULL AUTO_INCREMENT,
            postId bigint(100) DEFAULT 0, 
            productId varchar(255) DEFAULT '', 
            availablePublicationCount bigint(100) DEFAULT 0,
            contextualPricing MEDIUMTEXT DEFAULT '', 
            createdAt datetime DEFAULT '0000-00-00 00:00:00',
            customProductType MEDIUMTEXT DEFAULT '', 
            defaultCursor MEDIUMTEXT DEFAULT '', 
            description LONGTEXT DEFAULT '',
            descriptionHtml LONGTEXT DEFAULT '', 
            featuredImage MEDIUMTEXT DEFAULT '', 
            featuredMedia MEDIUMTEXT DEFAULT '', 
            handle varchar(255) DEFAULT '', 
            hasOnlyDefaultVariant tinyint(1) DEFAULT 0,
            hasOutOfStockVariants tinyint(1) DEFAULT 0,
            isGiftCard tinyint(1) DEFAULT 0,
            legacyResourceId varchar(255) DEFAULT '', 
            mediaCount bigint(100) DEFAULT 0,
            metafield MEDIUMTEXT DEFAULT '', 
            onlineStorePreviewUrl MEDIUMTEXT DEFAULT '', 
            onlineStoreUrl MEDIUMTEXT DEFAULT '', 
            options MEDIUMTEXT DEFAULT '', 
            priceRangeV2 MEDIUMTEXT DEFAULT '', 
            privateMetafield MEDIUMTEXT DEFAULT '', 
            productType MEDIUMTEXT DEFAULT '', 
            publicationCount bigint(100) DEFAULT 0,
            publishedAt datetime DEFAULT '0000-00-00 00:00:00',
            publishedOnCurrentPublication varchar(255) DEFAULT '', 
            publishedOnPublication MEDIUMTEXT DEFAULT '', 
            requiresSellingPlan tinyint(1) DEFAULT 0,
            resourcePublicationOnCurrentPublication MEDIUMTEXT DEFAULT '', 
            sellingPlanGroupCount bigint(100) DEFAULT 0,
            seo MEDIUMTEXT DEFAULT '', 
            standardizedProductType MEDIUMTEXT DEFAULT '', 
            status varchar(255) DEFAULT '', 
            tags MEDIUMTEXT DEFAULT '', 
            title MEDIUMTEXT DEFAULT '', 
            totalInventory bigint(100) DEFAULT 0,
            totalVariants bigint(100) DEFAULT 0,
            tracksInventory tinyint(1) DEFAULT 0,
            translations MEDIUMTEXT DEFAULT '', 
            updatedAt datetime DEFAULT '0000-00-00 00:00:00',
            vendor MEDIUMTEXT DEFAULT '', 
            collections LONGTEXT DEFAULT '',
			PRIMARY KEY  (id)
		) ENGINE=InnoDB $collate";
    }
}
