<?php

namespace ShopWP\DB;

use ShopWP\Utils;

if (!defined('ABSPATH')) {
    exit();
}

class Tags extends \ShopWP\DB
{
    public $table_name_suffix;
    public $table_name;
    public $version;
    public $primary_key;
    public $lookup_key;
    public $cache_group;
    public $type;

    public function __construct()
    {
        $this->table_name_suffix = SHOPWP_TABLE_NAME_TAGS;
        $this->table_name = $this->get_table_name();
        $this->version = '1.0';
        $this->primary_key = 'id';
        $this->lookup_key = 'tagId';
        $this->cache_group = 'shopwp_db_tags';
        $this->type = 'tag';
    }

    public function get_columns()
    {
        return [
            'id' => '%d',
            'productId' => '%s',
            'tag' => '%s',
        ];
    }

    public function cols_that_should_remain_ints()
    {
        return ['id'];
    }

    public function modify_options(
        $shopify_item,
        $item_lookup_key = SHOPWP_PRODUCTS_LOOKUP_KEY
    ) {
        return [
            'item' => $shopify_item,
            'item_lookup_key' => $item_lookup_key,
            'item_lookup_value' => $shopify_item->id,
            'prop_to_access' => 'tags',
            'change_type' => 'tag',
        ];
    }

    public function mod_before_change($tag) {

        $tag_copy = $this->copy($tag);

        if (isset($tag_copy->productId)) {
            $tag_copy->productId = \str_replace("gid://shopify/Product/", "", $tag_copy->productId);
        }

        unset($tag_copy->__parentId);

        return $tag_copy;

    }

    public function insert_tag($tag)
    {
        return $this->insert($tag);
    }

    public function delete_tag($tag)
    {
        return $this->delete_rows(
            'tag',
            $this->get_lookup_value($tag)
        );
    }

    public function get_tags_from_product_id($product_id)
    {

        $tags_raw = $this->get_rows(SHOPWP_PRODUCTS_LOOKUP_KEY, $product_id);

        $only_tags = array_unique(array_map(function($tag) {
            return $tag->tag;
        }, $tags_raw));

        return $only_tags;
    }

    public function get_product_ids_from_tags($tags)
    {
        return $this->select_in_col('product_id', 'tag', $tags);
    }

    public function delete_tags_from_product_id($product_id)
    {
        return $this->delete_rows(SHOPWP_PRODUCTS_LOOKUP_KEY, $product_id);
    }

    public function get_id_from_shopify($product)
    {
        if (Utils::has($product, 'id')) {
            return $product->id;
        } else {
            return 0;
        }
    }

    public function construct_tag_model($tag, $product = 0, $post_id = 0)
    {
        return [
            'tagId' => 0,
            'productId' => $this->get_id_from_shopify($product),
            'tag' => $tag,
        ];
    }

    public function construct_tags_for_insert($product, $post_id = 0)
    {
        $results = [];
        $tags = Utils::comma_list_to_array($product->tags);

        foreach ($tags as $tag) {
            $results[] = Utils::convert_array_to_object(
                $this->add_tag_id_to_tag(
                    $this->construct_tag_model($tag, $product, $post_id)
                )
            );
        }

        return $results;
    }

    public function create_tag_id($tag)
    {
        return Utils::hash_static_num($tag['product_id'] . $tag['tag']);
    }

    public function add_tag_id_to_tag($tag)
    {
        $tag_id_hash = $this->create_tag_id($tag);
        $tag['tag_id'] = $tag_id_hash;

        return $tag;
    }

    public function create_table_query($table_name = false)
    {
        if (!$table_name) {
            $table_name = $this->table_name;
        }

        $collate = $this->collate();

        return "CREATE TABLE $table_name (
			id bigint(100) unsigned NOT NULL AUTO_INCREMENT,
			productId longtext DEFAULT '',
			tag mediumtext DEFAULT '',
			PRIMARY KEY  (id)
		) ENGINE=InnoDB $collate";
    }
}
