<?php

namespace ShopWP\DB;

use ShopWP\Utils;
use ShopWP\Utils\Pricing;

if (!defined('ABSPATH')) {
    exit();
}

class Variants extends \ShopWP\DB
{

    public $table_name_suffix;
    public $table_name;
    public $version;
    public $primary_key;
    public $lookup_key;
    public $cache_group;
    public $type;

    public function __construct()
    {
        $this->table_name_suffix = SHOPWP_TABLE_NAME_VARIANTS;
        $this->table_name = $this->get_table_name();
        $this->version = '1.0';
        $this->primary_key = 'id';
        $this->lookup_key = 'variantId';
        $this->cache_group = 'shopwp_db_variants';
        $this->type = 'variant';
    }

    public function get_columns()
    {
        return [
            'id'                            => '%s',
            'variantId'                     => '%s',
            'productId'                     => '%s',
            'availableForSale'              => '%d',
            'barcode'                       => '%s',
            'compareAtPrice'                => '%s',
            'contextualPricing'             => '%s',
            'createdAt'                     => '%s',
            'defaultCursor'                 => '%s',
            'deliveryProfile'               => '%s',
            'displayName'                   => '%s',
            'fulfillmentService'            => '%s',
            'fulfillmentServiceEditable'    => '%s',
            'image'                         => '%s',
            'inventoryItem'                 => '%s',
            'inventoryPolicy'               => '%s',
            'inventoryQuantity'             => '%s',
            'legacyResourceId'              => '%s',
            'metafield'                     => '%s',
            'position'                      => '%d',
            'price'                         => '%s',
            'privateMetafield'              => '%s',
            'product'                       => '%s',
            'selectedOptions'               => '%s',
            'sellableOnlineQuantity'        => '%d',
            'sellingPlanGroupCount'         => '%d',
            'sku'                           => '%s',
            'taxCode'                       => '%s',
            'taxable'                       => '%d',
            'title'                         => '%s',
            'translations'                  => '%s',
            'updatedAt'                     => '%s',
            'weight'                        => '%f',
            'weightUni'                     => '%s'
        ];
    }

    public function cols_that_should_remain_ints()
    {
        return [
            'id',
            'position',
            'sellableOnlineQuantity',
            'sellingPlanGroupCount',
            'weight'
        ];
    }

    public function modify_options(
        $shopify_item,
        $item_lookup_key = SHOPWP_PRODUCTS_LOOKUP_KEY
    ) {
        return [
            'item' => $shopify_item,
            'item_lookup_key' => $item_lookup_key,
            'item_lookup_value' => $shopify_item->id,
            'prop_to_access' => 'variants',
            'change_type' => 'variant',
        ];
    }

    public function mod_before_change($variant)
    {
        $variant_copy = $this->copy($variant);
        $variant_copy = $this->maybe_rename_to_lookup_key($variant_copy);

        if (isset($variant_copy->productId)) {
            $variant_copy->productId = \str_replace("gid://shopify/Product/", "", $variant_copy->productId);
        }

        if (isset($variant_copy->variantId)) {
            $variant_copy->variantId = \str_replace("gid://shopify/ProductVariant/", "", $variant_copy->variantId);
        }

        unset($variant_copy->__parentId);

        return $variant_copy;

    }

    public function insert_variant($variant)
    {
        return $this->insert($variant);
    }

    public function update_variant($variant)
    {
        return $this->update(
            $this->lookup_key,
            $this->get_lookup_value($variant),
            $variant
        );
    }

    public function delete_variant($variant)
    {
        return $this->delete_rows(
            $this->lookup_key,
            $this->get_lookup_value($variant)
        );
    }

    public function delete_variants_from_product_id($product_id)
    {
        return $this->delete_rows(SHOPWP_PRODUCTS_LOOKUP_KEY, $product_id);
    }

    public function get_variants_from_product_id($product_id)
    {
        return $this->get_rows(SHOPWP_PRODUCTS_LOOKUP_KEY, $product_id);
    }

    public function unserialize_variants($variants) {
        return array_map(function($variant) {

            $variant->selectedOptions = maybe_unserialize($variant->selectedOptions);
            $variant->product = maybe_unserialize($variant->product);
            $variant->inventoryItem = maybe_unserialize($variant->inventoryItem);

            return $variant;

        }, $variants);
    }

    public function create_table_query($table_name = false)
    {
        if (!$table_name) {
            $table_name = $this->table_name;
        }

        $collate = $this->collate();

        return "CREATE TABLE $table_name (
            id bigint(100) unsigned NOT NULL AUTO_INCREMENT,
            variantId varchar(255) DEFAULT '',
            productId varchar(255) DEFAULT '',
            availableForSale tinyint(1) DEFAULT 0,
            barcode longtext DEFAULT '',
            compareAtPrice varchar(255) DEFAULT '',
            contextualPricing MEDIUMTEXT DEFAULT '',
            createdAt datetime DEFAULT '0000-00-00 00:00:00',
            defaultCursor mediumtext DEFAULT '',
            deliveryProfile mediumtext DEFAULT '',
            displayName mediumtext DEFAULT '',
            fulfillmentService longtext DEFAULT '',
            fulfillmentServiceEditable varchar(255) DEFAULT '',
            image longtext DEFAULT '',
            inventoryItem mediumtext DEFAULT '',
            inventoryPolicy varchar(255) DEFAULT '',
            inventoryQuantity varchar(255) DEFAULT '',
            legacyResourceId varchar(255) DEFAULT '',
            metafield mediumtext DEFAULT '',
            position bigint(100) DEFAULT 0,
            price varchar(255) DEFAULT '',
            privateMetafield mediumtext DEFAULT '',
            product mediumtext DEFAULT '',
            selectedOptions mediumtext DEFAULT '',
            sellableOnlineQuantity bigint(100) DEFAULT 0,
            sellingPlanGroupCount bigint(100) DEFAULT 0,
            sku longtext DEFAULT '',
            taxCode longtext DEFAULT '',
            taxable tinyint(1) DEFAULT 0,
            title mediumtext DEFAULT '',
            translations mediumtext DEFAULT '',
            updatedAt datetime DEFAULT '0000-00-00 00:00:00',
            weight decimal(20,4) DEFAULT 0.0,
            weightUni varchar(255) DEFAULT '',
			PRIMARY KEY  (id)
		) ENGINE=InnoDB $collate";
    }
}
