<?php

namespace ShopWP\Processing;

if (!defined('ABSPATH')) {
    exit();
}

use ShopWP\Utils;
use ShopWP\Utils\Data as Utils_Data;
use ShopWP\Utils\Server;
use ShopWP\Transients;

class Items extends \ShopWP\Processing\Vendor_Background_Process
{

    protected $action = 'shopwp_background_processing_items';
    public $DB_Settings_Syncing;
    public $DB_Settings_General;
    public $DB_Products;
    public $compatible_charset;
    public $CPT_Model;
    public $Media_Uploader;
    public $DB_Images;
    public $DB_Metafields;
    public $DB_Options;
    public $DB_Variants;
    public $DB_Tags;
    public $DB_Customers;
    public $DB_Orders;
    public $DB_Collections;
    public $selective_sync;
    public $settings;
    public $compatible_charsets;
    public $existing_attachments;
    public $user_id;
    public $DB;
    
    public function __construct(
        $DB_Settings_Syncing,
        $DB_Settings_General,
        $DB_Products,
        $DB_Collections,
        $DB_Images,
        $DB_Metafields,
        $DB_Options,
        $DB_Variants,
        $DB_Tags,
        $DB_Customers,
        $DB_Orders,
        $CPT_Model,
        $Media_Uploader
    ) {

        $this->DB                           = $DB_Settings_Syncing; // used only for readability
        $this->DB_Settings_Syncing          = $DB_Settings_Syncing;
        $this->DB_Settings_General          = $DB_Settings_General;
        $this->DB_Products                  = $DB_Products;
        $this->DB_Collections               = $DB_Collections;
        $this->DB_Images                    = $DB_Images;
        $this->DB_Metafields                = $DB_Metafields;
        $this->DB_Options                   = $DB_Options;
        $this->DB_Variants                  = $DB_Variants;
        $this->DB_Tags                      = $DB_Tags;
        $this->DB_Customers                 = $DB_Customers;
        $this->DB_Orders                    = $DB_Orders;
        $this->CPT_Model                    = $CPT_Model;
        $this->Media_Uploader               = $Media_Uploader;
        
        $this->selective_sync               = $this->DB_Settings_General->selective_sync_status();
        $this->settings                     = $this->DB_Settings_General->get();
        $this->compatible_charsets          = true;
        $this->existing_attachments         = false;
        $this->user_id                      = false;

        parent::__construct($DB_Settings_Syncing);

    }

    public function get_existing_attachments()
    {
        $existing_attachments = \wp_cache_get('shopwp_data_sync_existing_attachments');

        if ($existing_attachments) {
            return $existing_attachments;
        }

        $all_images_meta = $this->DB_Images->get_all_plugin_attachments_meta();

        if (!empty($all_images_meta)) {
            \wp_cache_add('shopwp_data_sync_existing_attachments', $all_images_meta);
        }

        return $all_images_meta;
    }

    public function process($data, $processing_manually = false)
    {

        if ($processing_manually) {

            $this->DB_Settings_Syncing->update_col('is_syncing', 1);
            $this->DB_Settings_Syncing->update_col('expire_sync_from_server', 1);
            
            $data = [json_encode(Utils::convert_object_to_array($data))];
        }

        if (!$this->DB_Settings_Syncing->is_syncing()) {

            $msg = 'Syncing has ended early, skipping process ...';

            $this->DB_Settings_Syncing->log($msg, 'warning');
            $this->DB_Settings_Syncing->save_warning($msg);

            return;

        }

        $this->existing_attachments = $this->get_existing_attachments();

        $this->selective_sync = $this->DB_Settings_General->selective_sync_status();

        $items_per_chunk = $this->DB_Settings_General->get_items_per_request();
        
        /*
        
        This does not chunk products specifically. This chunks "items", which could be product images,
        product variants, metafields, etc.
        
        */
       
        $chunks = Utils_Data::chunk_data($data, $items_per_chunk);

        $this->DB_Settings_Syncing->set_current_syncing_step_text('Saving data in chunks to be processed ...');

        foreach ($chunks as $chunk) {

            // $chunk_hash = Utils::hash($chunk, true);
            $this->DB_Settings_Syncing->log(__('Saving new chunk of data', 'shopwp') . '(' . count($chunk) . ')');

            foreach ($chunk as $item_in_chunk) {
                $this->push_to_queue($item_in_chunk);
            }

            $this->save();

        }

        $this->DB_Settings_Syncing->set_current_syncing_step_text(__('Processing data ...', 'shopwp'));
        $this->DB_Settings_Syncing->log(__('Processing data ... ', 'shopwp'));

        $this->dispatch();
        
    }

    public function save() {
        parent::save();
        $this->data = [];
        return $this;
    }


    public function on_batch_delete($batch_key) {
        $this->DB_Settings_Syncing->log(__('Cleared finished batch', 'shopwp'));
        return $batch_key;
    }

    public function get_existing_image($src) {

        $found = \explode('/products/', $src);

        $heeey = \explode('?v=', $found[1]);

        $filename = $heeey[0];

        global $wpdb;

        $query = "SELECT post_id FROM $wpdb->postmeta WHERE meta_key = '_wp_attached_file' AND meta_value LIKE '%" . $filename . "%'";
        
        return $wpdb->get_var($query);
    }

    public function process_images($image) {

        $image->productId = $image->__parentId;

        $numeric_id     = Utils::create_numeric_id($image->__parentId);
        $post_id        = $this->DB_Products->get_post_id_from_product_id($numeric_id);
        $product        = $this->DB_Products->get_product($post_id);

        if (empty($product)) {
            return false;
        }

        $img_is_featured = \maybe_unserialize($product->featuredImage);

        if ($img_is_featured->url === $image->image->url) {
            $is_featured = true;
        } else {
            $is_featured = false;
        }

        $image_result = $this->Media_Uploader->upload_image($image, $this->existing_attachments, $post_id, $is_featured);

        $insert_result = $this->DB_Images->insert($image);

        if (is_wp_error($insert_result)) {
            $this->DB_Settings_Syncing->log(__('Error while inserting image: ', 'shopwp') . $image->url . '. Error message: ' . $insert_result->get_error_message(), 'error');
        }

    }

    public function process_metafields($metafield) {

        $metafield->metafieldId = $metafield->id;
        $metafield->productId = $metafield->__parentId; 

        if ($metafield->namespace !== 'global') {

            $product_id = Utils::create_numeric_id($metafield->__parentId);

            $post_id = $this->DB_Products->get_post_id_from_product_id($product_id);

            $updated_post_meta = update_post_meta($post_id, $metafield->key, $metafield->value);

            $insert_result = $this->DB_Metafields->insert($metafield);

            if (is_wp_error($insert_result)) {
                $this->DB_Settings_Syncing->log($insert_result->get_error_message(), 'error');
            }

        }

    }

    public function process_options($product) {

        if (empty($product->options)) {
            return [];
        }

        foreach ($product->options as $option) {

            $option->productId = $product->id;

            $insert_result = $this->DB_Options->insert($option);

            if (is_wp_error($insert_result)) {
                $this->DB_Settings_Syncing->log(__('Error while inserting product option: ', 'shopwp') . $option->name . '. Error message: ' . $insert_result->get_error_message(), 'error');
            }
        }

    }

    public function process_variants($variant) {
        
        $variant->productId = $variant->__parentId;

        $insert_result = $this->DB_Variants->insert($variant);

        if (is_wp_error($insert_result)) {
            $this->DB_Settings_Syncing->log(__('Error while inserting product variant: ', 'shopwp') . $variant->id . '. Error message: ' . $insert_result->get_error_message(), 'error');
        }

    }

    public function process_customers($customer) {

        $insert_result = $this->DB_Customers->insert($customer);

        if (is_wp_error($insert_result)) {
            $this->DB_Settings_Syncing->log(__('Error while inserting customer: ', 'shopwp') . $customer->id . '. Error message: ' . $insert_result->get_error_message(), 'error');
        }

    }

    public function process_orders($order) {

        $insert_result = $this->DB_Orders->insert($order);

        if (is_wp_error($insert_result)) {
            $this->DB_Settings_Syncing->log(__('Error while inserting order: ', 'shopwp') . $order->id . '. Error message: ' . $insert_result->get_error_message(), 'error');
        }

    }

    public function process_tags($product) {

        if (empty($product->tags)) {
            return [];
        }

        foreach ($product->tags as $tag) {

            $tag_obj = new \stdClass();
            $tag_obj->productId = $product->id;
            $tag_obj->tag = $tag;
            
            $insert_result = $this->DB_Tags->insert($tag_obj);

            if (is_wp_error($insert_result)) {
                $this->DB_Settings_Syncing->log(__('Error while inserting product tag: ', 'shopwp') . $tag_obj->tag . '. Error message: ' . $insert_result->get_error_message(), 'error');
            }

        }

    }

    public function process_product_collections($product, $post_id) {
        
        if (\is_wp_error($post_id)) {
            return false;
        }

        $terms = get_the_terms($post_id, 'shopify_collections');

        $product->collections = \maybe_serialize($terms);

        $insert_result = $this->DB_Products->insert($product);

        if (is_wp_error($insert_result)) {
            $this->DB_Settings_Syncing->log(__('Error while inserting product: ', 'shopwp') . $product->title . ' for collections. Error message: ' . $insert_result->get_error_message(), 'error');
        }

    }

    public function process_product($product, $post_id) {

        if (\is_wp_error($post_id)) {
            $post_id = false;
        }

        // Only sync feat image if we're syncing images
        if ($this->selective_sync['images']) {

            if (!empty($product->featuredImage)) {
                $image_result = $this->Media_Uploader->upload_image($product->featuredImage, $this->existing_attachments, $post_id, true);
            }
            
        }

        $product->postId = $post_id;

        $insert_result = $this->DB_Products->insert($product);

        if (is_wp_error($insert_result)) {
            
            $this->DB_Settings_Syncing->log(__('Error while inserting product: ', 'shopwp') . $product->title . '. Error message: ' . $insert_result->get_error_message(), 'error');
            
            return $insert_result;

        }

        return $insert_result;

    }

    public function process_collection_item($collection, $post_id) {
        
        $collection->postId = $post_id;

        // Only sync feat image if we're syncing images
        if ($this->selective_sync['images']) {
            
            if (!empty($collection->image)) {

                $image_result = $this->Media_Uploader->upload_image($collection->image, $this->existing_attachments, $post_id);

                $collection->image->collectionId = $collection->id;

                $insert_result = $this->DB_Images->insert($collection->image);

                if (is_wp_error($insert_result)) {

                    if (isset($collection->url)) {
                        $url = $collection->url;
                    } else {
                        $url = 'Unknown';
                    }

                    $this->DB_Settings_Syncing->log(__('Error while inserting collection image: ', 'shopwp') . $url . '. Error message: ' . $insert_result->get_error_message(), 'error');
                }
                
            }
        }

        $insert_result = $this->DB_Collections->insert($collection);

        if (is_wp_error($insert_result)) {
            $this->DB_Settings_Syncing->log(__('Error while inserting product: ', 'shopwp') . $collection->title . '. Error message: ' . $insert_result->get_error_message(), 'error');
        }

    }

    public function is_item_type($data, $type) {

        if (empty($data)) {
            $this->DB_Settings_Syncing->log(__('Data is empty, skipping ...', 'shopwp'));
            return false;
        }

        if (property_exists($data, 'id') ) {

            if (Utils::str_contains($data->id, '/' . $type . '/')) {
                return true;
            }

        } else if (property_exists($data, 'image')) {
            if ($type === 'ProductImage') {
                return true;
            }
        }

        return false;
    }

    public function is_syncing_collection($sync_by_collections, $collection_title) {

        $found = false;

        foreach ($sync_by_collections as $collection) {
            if ($collection['title'] === $collection_title) {
                $found = true;
                break;
            }
        }

        return $found;
    }

    public function process_collection_single($settings, $item_decoded, $map_needed) {

        if (empty($settings)) {
            $settings = $this->DB_Settings_General->get();
        }

        if ($map_needed) {
            $item_decoded = $this->DB_Collections->normalize_collection_columns($item_decoded);
        }

        if ($settings->is_syncing_posts) {

            $post_id = $this->CPT_Model->insert_or_update_collection_post($item_decoded, $settings);

            if (\is_wp_error($post_id)) {
                $this->DB_Settings_Syncing->log($post_id->get_error_message(), 'error');
            }

        } else {
            $post_id = false;
        }

        $this->process_collection_item($item_decoded, $post_id);
        $this->DB_Settings_Syncing->log(__('Finished syncing collection: ', 'shopwp') . $item_decoded->title);
        
    }

    public function process_collection($item_decoded, $map_needed = false) {

        $settings = $this->settings;

        if ($settings->selective_sync_collections && !empty($item_decoded->publishedOnCurrentPublication)) {

            $sync_by_collections = \maybe_unserialize($settings->sync_by_collections);

            // Syncing all collections
            if (empty($sync_by_collections)) {

                $this->process_collection_single($settings, $item_decoded, $map_needed);

            // Only sync the collections selected within the ShopWP settings
            } else {

                $found = $this->is_syncing_collection($sync_by_collections, $item_decoded->title);

                if (!empty($found)) {
                    $this->process_collection_single($settings, $item_decoded, $map_needed);
                }

            }

        }
    }

    public function maybe_insert_product_post($product, $settings) {

        if ($settings->is_syncing_posts) {

            $post_id = $this->CPT_Model->insert_or_update_product_post($product, true, $settings);

            if (\is_wp_error($post_id)) {
                $this->DB_Settings_Syncing->log($post_id->get_error_message(), 'error');
            }

        } else {
            $post_id = false;
        }

        return $post_id;

    }

    public function should_sync_product($item) {
        return (isset($item->publishedOnCurrentPublication) && $item->publishedOnCurrentPublication) || $item->status === 'DRAFT';
    }

    /*

	Performs actions required for each item in the queue

	*/
    public function task($item)
    {

        if ($this->time_exceeded() || $this->memory_exceeded()) {
            $this->DB_Settings_Syncing->log(__('Time or memory exceeded, skipping ...', 'shopwp'), 'warning');            
            return $item;
        }

        // Stops background process if syncing stops
        if (!$this->DB_Settings_Syncing->is_syncing()) {
            $this->complete();
            return false;
        }

        if (empty($item)) {
            $this->DB_Settings_Syncing->log(__('Empty product found within process task, skipping ...', 'shopwp'), 'warning');
            return false;
        }
        
        $item_decoded = json_decode($item);

        if (isset($item_decoded->status) && $item_decoded->status === 'DRAFT' && $this->DB_Settings_General->col('skip_draft_products', 'bool')) {
            $this->DB_Settings_Syncing->log(__('Skipping draft product: ', 'shopwp') . $item_decoded->title, 'warning');
            $this->DB_Settings_Syncing->increment_current_amount('products');
            return false;
        }

        /*
        
        Because Shopify requires that we include the "id" param to the collections query, the returned
        data will have a single line with this ID. We need to count it towards our syncing and just return.
        
        */  
        if ($this->is_item_type($item_decoded, 'Collection')) {

            $this->process_collection($item_decoded);
            $this->DB_Settings_Syncing->increment_current_amount('products');

        } else if ($this->is_item_type($item_decoded, 'Customer')) {

            $this->process_customers($item_decoded);
            $this->DB_Settings_Syncing->increment_current_amount('products');

        } else if ($this->is_item_type($item_decoded, 'Order')) {

            $this->process_orders($item_decoded);
            $this->DB_Settings_Syncing->increment_current_amount('products');

        } else if ($this->is_item_type($item_decoded, 'ProductVariant')) {

            $this->process_variants($item_decoded);

            $this->DB_Settings_Syncing->log(__('Finished syncing product variant: ', 'shopwp') . $item_decoded->displayName);

            $this->DB_Settings_Syncing->increment_current_amount('products');

        } else if ($this->is_item_type($item_decoded, 'Metafield')) {

            $this->process_metafields($item_decoded);

            $this->DB_Settings_Syncing->log(__('Finished syncing product metafield: ', 'shopwp') . $item_decoded->key);

            $this->DB_Settings_Syncing->increment_current_amount('products');


        } else if ($this->is_item_type($item_decoded, 'ProductImage')) {

            if (!$this->DB_Settings_General->col('sync_featured_images_only', 'bool')) {
                $this->process_images($item_decoded);
            }

            $this->DB_Settings_Syncing->increment_current_amount('products');

        } else if ($this->is_item_type($item_decoded, 'Product')) {

            if (!isset($item_decoded->status)) {
                $this->DB_Settings_Syncing->increment_current_amount('products');
                return false;
            }

            if ($item_decoded->status === 'ARCHIVED') {

                // Check if product is synced, if so, remove it
                $numeric_id = Utils::create_numeric_id($item_decoded->id);

                $post_id = $this->CPT_Model->get_post_id_by_item_id($numeric_id);

                if ($post_id) {

                    $images_deleted_resp    = $this->DB_Images->delete_images_from_product_id($numeric_id, $post_id);
                    $product_deleted_resp   = $this->DB_Products->delete_products_from_product_id($numeric_id);
                    $options_deleted_resp   = $this->DB_Options->delete_options_from_product_id($numeric_id);
                    $variants_deleted_resp  = $this->DB_Variants->delete_variants_from_product_id($numeric_id);
                    $tags_deleted_resp      = $this->DB_Tags->delete_tags_from_product_id($numeric_id);
                    $deleted                = \wp_delete_post($post_id, true);
                    
                }

                $this->DB_Settings_Syncing->increment_current_amount('products');

                return false;
            }
            
            if ( $this->should_sync_product($item_decoded) ) {

                $post_id = $this->maybe_insert_product_post($item_decoded, $this->settings);

                $this->process_product($item_decoded, $post_id);

                $this->DB_Settings_Syncing->log(__('Finished syncing product data for: ', 'shopwp') . $item_decoded->title);

                if ($this->selective_sync['options']) {
                    $this->process_options($item_decoded);
                    $this->DB_Settings_Syncing->log(__('Finished syncing product options for: ', 'shopwp') . $item_decoded->title);
                }

                if ($this->selective_sync['tags']) {
                    $this->process_tags($item_decoded);
                    $this->DB_Settings_Syncing->log(__('Finished syncing product tags for: ', 'shopwp') . $item_decoded->title);
                }

                if ($this->selective_sync['collections']) {
                    $this->process_product_collections($item_decoded, $post_id);
                    $this->DB_Settings_Syncing->log(__('Finished syncing product collections for: ', 'shopwp') . $item_decoded->title);
                }

            }

            $this->DB_Settings_Syncing->increment_current_amount('products');

        }  else {

            $warning_message = $this->sanitize_missing_type_warning($item_decoded);

            $this->DB_Settings_Syncing->log($warning_message, 'warning');
            $this->DB_Settings_Syncing->increment_current_amount('products');

        }         

        return false;
        
    }

    public function sanitize_missing_type_warning($item_decoded) {

        if (property_exists($item_decoded, 'id')) {
            $stuff = $item_decoded->id;

        } else if (property_exists($item_decoded, '__parentId') && property_exists($item_decoded, 'mediaContentType')) {
            $stuff = $item_decoded->mediaContentType . ' assigned to: ' . $item_decoded->__parentId;
        } else {
            $stuff = $item;
        }
        
        return __('No type found for: ', 'shopwp') . \serialize($stuff);
    }

    public function after_queue_item_removal($product)
    {
    }

    public function fire_syncing_complete_webhook() {
        \wp_remote_post(\get_rest_url(null, 'shopwp/v1/syncing/complete'), [
            'body' => [
                'data' => __('Hey there! Data will be coming in a future ShopWP release.', 'shopwp'),
            ]
        ]);
    }

    public function complete()
    {

        parent::complete();

        $syncing_current_amounts_products   = $this->DB_Settings_Syncing->col('syncing_current_amounts_products', 'int');
        $syncing_totals_products            = $this->DB_Settings_Syncing->col('syncing_totals_products', 'int');
        $expire_sync_from_server            = $this->DB_Settings_Syncing->col('expire_sync_from_server', 'bool');

        if ($syncing_current_amounts_products > $syncing_totals_products) {
            $this->DB_Settings_Syncing->log(__('Current amount is more than total, forcing them to be the same.', 'shopwp'), 'warning');
            $this->DB_Settings_Syncing->update_col('syncing_totals_products', $syncing_current_amounts_products);
            $this->DB_Settings_Syncing->update_col('syncing_totals_media', $syncing_current_amounts_products);
        }

        if ($expire_sync_from_server) {
            $this->DB_Settings_Syncing->update_col('expire_sync_from_server', false);
        }

        if ($this->DB_Settings_Syncing->is_syncing()) {
            $this->DB_Settings_Syncing->log('---------------------');
            $this->DB_Settings_Syncing->log(__('✓ Syncing complete', 'shopwp'));
            $this->DB_Settings_Syncing->log('---------------------');
            $this->DB_Settings_Syncing->expire_sync();
            $this->fire_syncing_complete_webhook();
        }

    }

    public function init() {

    }
}